procedure ccd48proc (input,imagetyp,ccdident,isdir)

# Main input parameters
string	input	=""		{prompt="List with images to process"}
string	imagetyp=""		{prompt="Image type",
				 enum="bias|dark|flat|sky|object"}
string	ccdident="auto"		{prompt="CCD detector"}
string	isdir	="ccd48$lib/CCD48/" {prompt="Instrument specific directory"}
string	subskey ="filter"	{prompt="Subset identifier header keyword"}
string	expkey	="exptime"	{prompt="Exposure time header keyword"}
string	drkkey	="darktime"	{prompt="Dark time header keyword"}
real	maxval	=32768.		{prompt="Maximum valid (linear) pixel value"}
string	logfile	="ccdproc.log"	{prompt="CCD48proc text log file"}
string	statlog	="ccdproc.stat"	{prompt="CCD48proc image statistics log file"}
string	zerolog	="biassub.log"	{prompt="Text log file bias subtraction"}
string	darklog	="darkcor.log"	{prompt="Text log file dark correction"}
string	flatlog	="flatcor.log"	{prompt="Text log file flatfielding"}
string	comblog	="combine.log"	{prompt="Text log file frame combination"}
bool	verbose	=yes		{prompt="Print verbose messages screen?"}

# Operation selecting parameters (all defaults except 'surface' are no)
bool	getstat	=no		{prompt="Log image statistics to statlog?"}
bool	wcsinit	=no		{prompt="Initialize WCS header keywords?"}
bool	fixpix	=no		{prompt="Fix bad CCD lines and columns?"}
bool	zerocor	=no		{prompt="Correction bias using zero image?"}
string	zero	="../BIAS"	{prompt="Zero level calibration image"}
bool	oversca	=no		{prompt="Subtract bias using overscan strip?"}
string	biassec	=""		{prompt="Overscan section"}
bool	trim	=no		{prompt="Trim the image?"}
string	trimsec	=""		{prompt="Data/trim section"}
bool	darkcor	=no		{prompt="Apply dark count correction?"}
string	dark	="../DARK"	{prompt="Dark count calibration image"}
bool	dmean	=no		{prompt="Use mean of dark count image?"}
bool	scatcor	=no		{prompt="Apply correction for scattered light?"}
string	scat	="../SCAT"	{prompt="Root of scattered light image name(s)"}
bool	shutcor	=no		{prompt="Apply shutter correction?"}
string	shut	="../SHUT"	{prompt="Root of shutter correction image name"}
bool	respfit	=no		{prompt="Fit response function?"}
bool	surface	=yes		{prompt="Fit 2-D surface i.s.o. 1-D lines?"}
int	order	=31		{prompt="Order of response fit"}
bool	flatcor	=no		{prompt="Apply flatfield correction?"}
string	flat	="../RESPONS"	{prompt="Root of flatfield image name(s)"}
bool	illumcor=no		{prompt="Apply illumination correction?"}
string	illum	="../ILLUM"	{prompt="Root of illumination image name(s)"}
bool	combine	=no		{prompt="Combine multiple calibration frames?"}
bool	normaliz=no		{prompt="Normalize combined flat/illum?"}
bool	modscat	=no		{prompt="Model scattered light (beware!)?"}
bool	remove	=no		{prompt="Delete input images after combination?"}
bool	copyem	=no		{prompt="Copy, not rename, processed frames?"}
int	maxcomb	=INDEF		{prompt="Max # of images to combine at once",
				 min=7,max=512}
string	rejcomb	="avsigclip"	{prompt="Rejection for image combination",
				 enum="avsigclip|minmax|none"}
string	version	="V2.12-LOCAL  06-Jul-2004/RAJ"	{prompt="Task version"}

# @(#) task ccd48.ccd48proc      Author: R.A. Jansen -- February 05 1996
# @(#) 
# @(#) Task to do the primary reduction of CCD image date of the AndyCam
# @(#) (one-shooter) camera (1994--1998) at the FLWO 48inch telescope at
# @(#) Mt. Hopkins.  The primary reduction consists of the fixing of bad
# @(#) pixels, correction for structure in the bias,  subtraction of the
# @(#) biaslevel as measured in the overscan strip,  subtraction of dark
# @(#) current (by subtracting either a scaled dark image  or the scaled
# @(#) median thereof),  flatfielding and  illumination correction.  The
# @(#) 'master'  calibration frames may also be produced with this task.
# @(#) Relevant information is logged to file  and to the image headers.
# @(#) 
# @(#) This task calls various subtasks in the 'ccd48' package, and also
# @(#) uses a host of 'rjtools' tasks (e.g. gdate, iwc, rdlist, imgstat,
# @(#) borderstat, chkimg, cparse, rpparse, ...)
# @(#)
# @(#) Package CCD48 consists of (sub)tasks:  ccd48proc,tdefs48,itest48,
# @(#)  ptest48,getid48,(gnpix48),gimst48,(gprst48),(gsubs48),(rmsub48),
# @(#)  rnimg48,wcsin48,fixpx48,zrcor48,ovsca48,dkcor48,shcor48,fresp48,
# @(#)  ftcor48,ilcor48,sccor48,(fstar48),zcomb48, fcomb48, fnorm48  and
# @(#)  scmod48,
# @(#) where parentheses indicate that the sub-task is obsolete (ie, not
# @(#) called any longer by 'ccd48proc').
# @(#) 
# @(#) <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
# @(#) Revision history:
# @(#) Sep 17 1996 -- Added CCD description for  FLWO 60inch + FAST  and
# @(#)                replaced old task 'fastproc' with 'ccd48proc'. RAJ
# @(#) Feb 04 2000 -- Added CCD description for ESO 3.6m + EFOSC1.  RAJ.
# @(#) Mar 06 2002 -- Added CCD description for Mt.Graham VATT+ccd. RAJ.
# @(#) Nov 05 2002 -- Added subtask 'wcsin48' to initalize WCS keywords.
# @(#) Aug 13 2003 -- Rewrite/clean-up of the  'ccd48'  package in order
# @(#) --Aug 30 2003  to streamline checks and processing and generalize
# @(#)                its application to additional telescopes. Also, it
# @(#)                is now possible to control  the reduction  via the
# @(#)                image headers  by listing the action keyword  with
# @(#)                an associated value of 'PERFORM'.  Action keywords
# @(#)                are:  WCSINIT, FIXPIX, ZEROCOR, OVERSCAN, DARKCOR,
# @(#)                (SCATCOR), (SHUTCOR), FLATCOR, ILLUMCOR, BIASCOMB,
# @(#)                DARKCOMB, FLATCOMB,  SKYCOMB,  and  NORMALIZ.  The
# @(#)                appropriate calibration images,  at present, still
# @(#)                need to be defined using the task parameters,  but 
# @(#)                this may change in future. RAJ.
# @(#) Sep 26 2003 -- Changed sub-task SHUTCOR from dummy task to proper
# @(#)                one, and added module RESPFIT to allow fitting the
# @(#)                pixel-to-pixel response prior to combining flats.
# @(#) Mar 12 2004 -- Added 'surface' switch for task  RESPFIT  to allow
# @(#)                switching between 2-D and 1-D fitting, and 'order'
# @(#)                parameter for the bi-cubic spline surface fit. For
# @(#)                1-D fitting the order is fixed to 3. RAJ.
# @(#) Mar 15 2004 -- Added 'rejcomb' switch to allow "minmax" or "none"
# @(#)                instead of "avsigclip" rejection in 'fcomb48'. RAJ
# @(#) Apr 16 2004 -- Fixed minor bug: filter names are now properly ap- 
# @(#)                pended to images names  for image types other than 
# @(#)                ZERO or DARK.  Changed format of package parameter
# @(#)                'version' to distinguish system-wide  and possible
# @(#)                locally installed modified versions. RAJ.
# @(#) May 11 2004 -- Fixed bug in logic (|| -> &&) that appended 'D' to
# @(#)                frame names in case of BIASes and DARKs. Also made
# @(#)                minor format change to output log file. RAJ+AM.
# @(#) Jun 25 2004 -- Added CCD description for Magellan I + IMACS. RAJ.
# @(#) Jul  6 2004 -- Expanded support for header-directed primary image
# @(#)                processing,  by adding keywords  CCDTRIM, BIASSEC,
# @(#)                TRIMSEC, (ZRCRIMAG,  DKCRIMAG, FTCRIMAG, ILCRIMAG,
# @(#)                SHCRIMAG). RAJ.
# @(#) >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

begin
	string	imlis, imgtyp, cimg, type, biasec, trmsec, imgsec, idccd
	string	fixfil, shtfil, isfil, sisdir, subset, exptim, drktim
	string	csub, ctyp, ccom, com2, cisdir, czero, cdark, cflat
	string	cillum, cscat, cshut, cmbr, dash
	int	nimg, ncol, nrow, maxn, i, ford, ld
	real	rdnoise, gain, secpix, texp, tdrk, smax
	bool	hdrproc, yyproc, anyproc, transp, first, ofirst, ifirst

#	Test whether all required packages are loaded...
	tdefs48 (ok=yes)
	if (!tdefs48.ok) { return }

#	Initialize variables...
	imlis	= input
	imgtyp	= imagetyp
	sisdir  = isdir
	cisdir  = sisdir
	isfil	= sisdir//"ccd.subsets"
	subset	= subskey
	exptim	= expkey
	drktim	= drkkey
	smax	= maxval
	czero   = zero
	biasec	= biassec
	trmsec	= trimsec
	cdark   = dark
	cshut   = shut
	ford    = order
	if (!surface) { ford = 3 }
	cflat   = flat
	cillum  = illum
	cscat   = scat
	maxn	= maxcomb
	cmbr    = rejcomb
	idccd	= "INDEF"
	fixfil	= "INDEF"
	shtfil  = "INDEF"
	rdnoise	= 0.
	gain	= 1.
	secpix	= 1.

#-----------------------------------------------------------------------------

#	Open logfile...
	sysinfo() ; gdate(utc=no)
	if ( !access(logfile) ) {
	   print ("## ===========================================================================", >> logfile)
	   print ("## "//ccd48.version//":  Automated CCD primary data reduction", >> logfile)
	   print ("## ===========================================================================", >> logfile)
	}

#	Test input and task parameters for conflicts or omissions...
	if (verbose) print ("Testing input and task parameters ... ")

	# Test whether the input list and images exist, and if so,
	# return the number of images and the image size...
	itest48 (imlis, logfile, ok=yes)
	if (!itest48.ok) {
	   print ("CCD48PROC: sub-task ITEST48 exited with error flag raised.")
	   return
	}
	nimg	= itest48.nimg
	ncol	= itest48.ncol
	nrow	= itest48.nrow
	hdrproc = itest48.hdrproc


#	Print opening message and log current task parameters...
	if ( substr(ccd48proc.version,7,11) == "LOCAL") {
	  print ("\n## CCD48PROC["//substr(ccd48proc.version,14,28)//"]:   "//sysinfo.username//"@"//sysinfo.host//"   "//gdate.tdate, >> logfile)
	} else {
	  print ("\n## CCD48PROC["//substr(ccd48proc.version,1,12)//"]:   "//sysinfo.username//"@"//sysinfo.host//"   "//gdate.tdate, >> logfile)
	}
	print ("## ---------------------------------------------------------------------------", >> logfile)
	print ("##   IRAF distribution:", >> logfile)
	print ("##     version =\""//envget("version")//"\"", >> logfile)
	print ("##   Generic and data dependent parameters:", >> logfile)
	print ("##     input   =\""//imlis//"\"", >> logfile)
	print ("##     isdir   =\""//cisdir//"\"", >> logfile)
	printf("##     imagetyp=\""//imgtyp//"\"%42tccdident=\""//ccdident//"\"\n", >> logfile)
	printf("##     subskey =\""//subset//"\"%42texpkey  =\""//exptim//"\"\n", >> logfile)
	printf("##     drkkey  =\""//drktim//"\"%42tmaxval  =\""//smax//"\"\n", >> logfile)
	printf("##     logfile =\""//logfile//"\"%42tstatlog =\""//statlog//"\"\n", >> logfile)
	printf("##     zerolog =\""//zerolog//"\"%42tdarklog =\""//darklog//"\"\n", >> logfile)
	printf("##     flatlog =\""//flatlog//"\"%42tcomblog =\""//comblog//"\"\n", >> logfile)
	if ( hdrproc && ( biasec == "" || trmsec == "" ) ) {
	   if ( biasec == "" && trmsec == "" ) {
	      printf("##     biassec =\"(@header)\"%42ttrimsec =\"(@header)\"\n", >> logfile)
	   } else {
	      if ( biasec == "" ) {
		 printf("##     biassec =\"(@header)\"%42ttrimsec =\""//trmsec//"\"\n", >> logfile)
	      } else {
		 printf("##     biassec =\""//biasec//"\"%42ttrimsec =\"(@header)\"\n", >> logfile)
	      }
	   }
	} else {
	   printf("##     biassec =\""//biasec//"\"%42ttrimsec =\""//trmsec//"\"\n", >> logfile)
	}
	printf("##     maxcomb =%d%42tverbose ="//verbose//"\n", maxn, >> logfile)
	print ("##   Processing switches:", >> logfile)
	if (hdrproc) {
	   print ("##     hdrproc =yes (image header keywords have precedence!)", >> logfile)
	   printf("##     getstat ="//getstat//"%25twcsinit =("//wcsinit//")%42tfixpix  =("//fixpix//")%59tzerocor =("//zerocor//")\n", >> logfile)
	   printf("##     oversca =("//oversca//")%25ttrim    =("//trim//")%42tdarkcor =("//darkcor//")%59tdmean   ="//dmean//"\n", >> logfile)
	   printf("##     shutcor =("//shutcor//")%25trespfit =("//respfit//")%42tsurface ="//surface//"%59torder   =%2d\n", ford, >> logfile)
	   printf("##     scatcor =("//scatcor//")%25tflatcor =("//flatcor//")%42tillumcor=("//illumcor//")%59tcombine =("//combine//")\n", >> logfile)
	   printf("##     normaliz=("//normaliz//")%25tmodscat ="//modscat//"%42tremove  ="//remove//"%59tcopyem  ="//copyem//"\n", >> logfile)
	   printf("##     rejcomb =\""//rejcomb//"\"\n", >> logfile)
	   print ("##   Calibration frame root names: (image header keywords have precedence!)", >> logfile)
	   printf("##     zero    =(\""//czero//"\")%42tdark    =(\""//cdark//"\")\n", >> logfile)
	   printf("##     shut    =(\""//cshut//"\")%42tscat    =(\""//cscat//"\")\n", >> logfile)
	   printf("##     flat    =(\""//cflat//"\")%42tillum   =(\""//cillum//"\")\n", >> logfile)
	} else {
	   printf("##     getstat ="//getstat//"%25twcsinit ="//wcsinit//"%42tfixpix  ="//fixpix//"%59tzerocor ="//zerocor//"\n", >> logfile)
	   printf("##     oversca ="//oversca//"%25ttrim    ="//trim//"%42tdarkcor ="//darkcor//"%59tdmean   ="//dmean//"\n", >> logfile)
	   printf("##     shutcor ="//shutcor//"%25trespfit ="//respfit//"%42tsurface ="//surface//"%59torder   =%2d\n", ford, >> logfile)
	   printf("##     scatcor ="//scatcor//"%25tflatcor ="//flatcor//"%42tillumcor="//illumcor//"%59tcombine ="//combine//"\n", >> logfile)
	   printf("##     normaliz="//normaliz//"%25tmodscat ="//modscat//"%42tremove  ="//remove//"%59tcopyem  ="//copyem//"\n", >> logfile)
	   printf("##     rejcomb =\""//rejcomb//"\"\n", >> logfile)
	   print ("##   Calibration frame root names:", >> logfile)
	   printf("##     zero    =\""//czero//"\"%42tdark    =\""//cdark//"\"\n", >> logfile)
	   printf("##     shut    =\""//cshut//"\"%42tscat    =\""//cscat//"\"\n", >> logfile)
	   printf("##     flat    =\""//cflat//"\"%42tillum   =\""//cillum//"\"\n", >> logfile)
	}
	print ("## ---------------------------------------------------------------------------", >> logfile)

	print ("\n## CCD48PROC:  "//sysinfo.sysmsg//"  "//gdate.tdate)

	if ( imgtyp == "bias"   ) imgtyp = "ZERO"
	if ( imgtyp == "dark"   ) imgtyp = "DARK"
	if ( imgtyp == "flat"   ) imgtyp = "FLAT"
	if ( imgtyp == "sky"    ) imgtyp = "SKY"
	if ( imgtyp == "object" ) imgtyp = "OBJECT"

#-----------------------------------------------------------------------------

	# Test task parameter definitions for consistency. Symbolic paths
	# are expanded to the full system paths upon exit of 'ptest48'...
	ptest48 (logfile, ncol, nrow, fixpix, zerocor, oversca, trim,
		darkcor, shutcor, flatcor, illumcor, scatcor, isdir=sisdir,
		isfil=isfil, zero=czero, biassec=biasec, trimsec=trmsec,
		dark=cdark, shut=cshut, flat=cflat, illum=cillum, scat=cscat,
		maxcomb=maxn, ok=yes)
	if (!ptest48.ok) {
	   print ("CCD48PROC: sub-task PTEST48 exited with error flag raised.")
	   return
	}
	cisdir = ptest48.isdir
	isfil  = ptest48.isfil
	zero   = ptest48.zero
	biasec = ptest48.biassec
	trmsec = ptest48.trimsec
	dark   = ptest48.dark
	shut   = ptest48.shut
	flat   = ptest48.flat
	illum  = ptest48.illum
	scat   = ptest48.scat
	maxn   = ptest48.maxcomb
	if ( trmsec == "" ) { imgsec = "[*,*]" } else { imgsec = trmsec }


#	Check detector identification and image size for first frame in
#	the input list (we already know that the images and files exist
#	and that all images in the list have the same size)...
	#if (verbose) print("Checking detector specifications ...")
	rdlist (imlis, 1, cline="")
	cimg = rdlist.cline
	getid48 (cimg, ccdident, sisdir, ncol, nrow, logfile, verbose,
		debug=no, ok=yes)
	if (!getid48.ok) {
	   print ("CCD48PROC: sub-task GETID48 exited with error flag raised.")
	   return
	}
	idccd	= getid48.idccd
	fixfil	= getid48.fixfile
	shtfil	= getid48.shtfile
	rdnoise	= getid48.rdnoise
	gain	= getid48.gain
	secpix	= getid48.secpix
	transp	= getid48.transp

#	Check for each image whether the subset (e.g. filter) is known
#	and create one list of images per subset encountered. Optionally
#	compute the image statistics in the trim- and bias-sections...
	gimst48 (imlis, isfil, imgtyp, hdrproc, nimg, ncol, nrow, smax, subset,
		exptim, logfile, getstat, statsec=imgsec, biassec=biasec,
		statlog=statlog, verbose=verbose, drkkey=drktim, ok=yes)
	if ( !gimst48.ok ) {
	   print ("CCD48PROC: sub-task GIMST48 exited with error flag raised.")
	   return
	}
	trmsec = gimst48.statsec
	biasec = gimst48.biassec
	drktim = gimst48.drkkey

#-----------------------------------------------------------------------------

#	Everything seems to be OK, so loop over all images in the input
#	list and process the images according to the selected switches...
	if (verbose) print ("Starting automated image processing ...\n")
	anyproc = no
	first   = yes
	ofirst	= yes
	ifirst  = yes
	for ( i = 1 ; i <= nimg ; i = i + 1 ) {
	   rdlist (imlis, i)
	   cimg = rdlist.cline

	   # Parse image name into path, root, and extension...
	   chkimg (cimg, "access", imtype="", verbose=no)
	   cimg = chkimg.impath//chkimg.imroot
	   ctyp = chkimg.imtype

	   # Read image type, subset and exposure time from the header...
	   hselect (cimg//ctyp, "imagetyp", "yes") | scan (type)
	   hselect (cimg//ctyp, subset, "yes") | scan (csub)
	   hselect (cimg//ctyp, exptim, "yes") | scan (texp)
	   hselect (cimg//ctyp, drktim, "yes") | scan (tdrk)

	   # Log image processing information to logfile and screen...
	   gdate(utc=yes)
	   printf("CCD48PROC Image="//cimg//ctyp//"[%d,%d]["//csub//"]["//type//"] ...\n", ncol, nrow)
	   printf("CCD48PROC Image="//cimg//ctyp//"[%d,%d]["//csub//"]["//type//"]:\n", ncol, nrow, >> logfile)
	   print ("  CC48PROC: "//gdate.fdate//" Starting primary frame processing", >> logfile)

	   # Append subset identifier to image name...
	   if ( type != "ZERO" && type != "DARK" ) {
	      imrename (cimg//ctyp, cimg//csub//ctyp, verbose-)
	      cimg = cimg//csub
	   }

	   # Add time stamp to image header marking start of processing...
	   ccom = "" ; hselect (cimg//ctyp, "cc48proc", "yes") | scan (ccom)
	   if ( ccom != "" ) {
	      #if (verbose) print ("    CC48PROC ... processing using header parameters enabled")
	      #print ("  CC48HPPR: "//gdate.fdate//" Processing using header parameters enabled", >> logfile)
	      if (verbose) print ("    CC48PROC ... header-directed processing enabled")
	      print ("  CC48HPPR: "//gdate.fdate//" Header-directed processing enabled", >> logfile)
	      hedit (cimg//ctyp, "cc48proc", gdate.fdate, add-, del-, ver-, show-, upd+)
	   } else {
	      if ( access("ccdpr.head") ) delete ("ccdpr.head", yes, verify-)
	      imheader (cimg//ctyp, longheader+, > "ccdpr.head")
	      print ("HISTORY -", >> "ccdpr.head")

	      ld = nint(0.5*(37-strlen(sysinfo.username)-strlen(sysinfo.host)))
	      dash = "------------"
              print ("HISTORY "//substr(dash,1,ld)//"  CCD48PROC["//sysinfo.version//"]  "//sysinfo.username//"@"//sysinfo.host//"  "//substr(dash,1,ld), >>"ccdpr.head")

	      print ("HISTORY -", >> "ccdpr.head")
	      print ("CC48PROC= '"//gdate.fdate//"' / Starting primary frame processing", >> "ccdpr.head")
	      mkheader (cimg//ctyp, "ccdpr.head", append-, verbose-)
	      delete ("ccdpr.head", yes, verify-)
	   }

	   yyproc= no

	   # Initialize WCS header keywords...
	   if (wcsinit) { ccom = "PERFORM" } else { ccom = "OMIT" }
	   hselect (cimg//ctyp, "wcsinit", "yes") | scan (ccom)
	   if ( ccom == "PERFORM" ) {
	      if ( type == "OBJECT" ) {
		 rnimg48 (cimg, ctyp, "w", copyem)
		 cimg = rnimg48.result
	      }
	      yyproc = yes
	   }
	   wcsin48 (ccom, cimg//ctyp, ncol, nrow, logfile, transp, verbose)

	   # Fix bad CCD lines and columns described in 'fixfil'...
	   if (fixpix) { ccom = "PERFORM" } else { ccom = "OMIT" }
	   hselect (cimg//ctyp, "fixpix", "yes") | scan (ccom)
	   if ( ccom == "PERFORM" ) {
	      if ( type == "OBJECT" ) {
		 rnimg48 (cimg, ctyp, "x", copyem)
		 cimg = rnimg48.result
	      }
	      yyproc = yes
	   }
	   fixpx48 (ccom, cimg, fixfil, logfile, verbose)

	   # Correct for structure in the bias level using a zero image...
	   if ( type != "ZERO" ) {
	      if (zerocor) { ccom = "PERFORM" } else { ccom = "OMIT" }
	      hselect (cimg//ctyp, "zerocor", "yes") | scan (ccom)
	      if ( ccom == "PERFORM" ) {
		 if ( type == "OBJECT" ) {
		    rnimg48 (cimg, ctyp, "z", copyem)
		    cimg = rnimg48.result
		 }
		 yyproc = yes
	         ofirst = no
	      }
	      zrcor48 (ccom, cimg, zero, logfile, zerolog, verbose, first)
	   }

	   # Subtract bias level as determined in the overscan region...
	   if (oversca) { ccom = "PERFORM" } else { ccom = "OMIT" }
	   hselect (cimg//ctyp, "overscan", "yes") | scan (ccom)
	   if ( ccom == "PERFORM" ) {
	      if ( type == "OBJECT" ) {
		 rnimg48 (cimg, ctyp, "b", copyem)
		 cimg = rnimg48.result
	      }
	      yyproc = yes
	   }
	   if (trim) { com2 = "PERFORM" } else { com2 = "OMIT" }
	   com2 = "" ; hselect (cimg//ctyp, "ccdtrim", "yes") | scan (com2)
	   if ( com2 == "PERFORM" ) { trim = yes }
	   ovsca48 (ccom, cimg, biasec, trim, trmsec, ncol, nrow, logfile,
			zerolog, verbose, ofirst)
	   imgsec = ovsca48.imagsec

	   # Subtract non-zero darkcurrent using a dark count image ...
	   if ( type != "ZERO" && type != "DARK" ) {
	      if (darkcor) { ccom = "PERFORM" } else { ccom = "OMIT" }
	      hselect (cimg//ctyp, "darkcor", "yes") | scan (ccom)
	      if ( ccom == "PERFORM" ) {
		 if ( type == "OBJECT" ) {
		    rnimg48 (cimg, ctyp, "d", copyem)
		    cimg = rnimg48.result
		 }
		 yyproc = yes
	      }
	      dkcor48 (ccom, cimg, tdrk, dark, dmean, logfile, darklog,
			verbose, first)
	   }

	   # Correct short exposure image for finite shutter time; at present,
	   # only implemented and tested for a diaphragm-type shutter at the
	   # Vatican Advanced Technology Telescope's 2kx2k CCD camera...
	   if ( type != "ZERO" && type != "DARK" ) {
	      if (shutcor) { ccom = "PERFORM" } else { ccom = "OMIT" }
	      hselect (cimg//ctyp, "shutcor", "yes") | scan (ccom)
	      if ( ccom == "PERFORM" ) {
		 if ( type == "OBJECT" ) {
		    rnimg48 (cimg, ctyp, "h", copyem)
		    cimg   = rnimg48.result
		 }
		 yyproc = yes
	      }
	      shcor48 (ccom, cimg, texp, shut, shtfil, logfile, flatlog,
			verbose, first)
	      if ( !shcor48.ok ) {
		 print ("CCD48PROC: task SHCOR48 exited with error flag raised.")
		 return
	      }
	   }

	   # Fit pixel-to-pixel response prior to flatfield combination...
	   if ( type == "FLAT" || type == "SKY" ) {
	      if (respfit && order > 1 && order < 50 ) {
		 ccom = "PERFORM"
	      } else { ccom = "OMIT" }
	      hselect (cimg//ctyp, "respfit", "yes") | scan (ccom)
	      fresp48 (ccom, cimg, ford, surface, logfile, flatlog, verbose, first)
	   }

	   if ( type == "OBJECT" ) {

	      # Apply flat field correction using response image...
	      if (flatcor) { ccom = "PERFORM" } else { ccom = "OMIT" }
	      hselect (cimg//ctyp, "flatcor", "yes") | scan (ccom)
	      if ( ccom == "PERFORM" ) {
		 rnimg48 (cimg, ctyp, "f", copyem)
		 cimg   = rnimg48.result
		 yyproc = yes
		 ifirst = no
	      }
	      ftcor48 (ccom, cimg, cflat, csub, logfile, flatlog, verbose, first)

	      # Apply illumination correction using illumination image...
	      if (illumcor) { ccom = "PERFORM" } else { ccom = "OMIT" }
	      hselect (cimg//ctyp, "illumcor", "yes") | scan (ccom)
	      if ( ccom == "PERFORM" ) {
		 rnimg48 (cimg, ctyp, "i", copyem)
		 cimg   = rnimg48.result
		 yyproc = yes
	      }
	      ilcor48 (ccom, cimg, illum, csub, logfile, flatlog,verbose,ifirst)

	      # Apply scattered light correction using calibration image...
	      if (scatcor) { ccom = "PERFORM" } else { ccom = "OMIT" }
	      hselect (cimg//ctyp, "scatcor", "yes") | scan (ccom)
	      if ( ccom == "PERFORM" ) {
		 rnimg48 (cimg, ctyp, "l", copyem)
		 cimg   = rnimg48.result
		 yyproc = yes
	      }
	      sccor48 (ccom, cimg, scat, csub, trmsec, logfile, verbose)

	   }

	   if (yyproc) {
	      anyproc = yes
	      gdate()
	      print ("  CC48DATE: "//gdate.fdate//" Primary frame processing done", >> logfile)
	   #} else {
	      # leave header unchanged; image may have been processed already!
	      #hedit (cimg//ctyp, "cc48proc", "", add-, del+, ver-, show-, upd+)
	      #hedit (cimg//ctyp, "cc48date", "", add-, del+, ver-, show-, upd+)
	   }

	   first  = no
	   ofirst = no
	   ifirst = no

	# goto next image...
	}

	if (verbose) printf("\n")


	# Now start combining the processed images for types bias, dark, flat
	# and sky if anyproc=yes; note that, here, the *last* image in the
	# input list determines whether or not to combine separate calibration
	# images when processing is directed through header keyword values,
	# rather than through the task parameters.

	if ( anyproc && ( imgtyp == "ZERO" || imgtyp == "DARK" ) ) {
	   # Combine the processed bias or dark frames ...
	   if (combine) { ccom = "PERFORM" } else { ccom = "OMIT" }
	   if ( imgtyp == "ZERO" ) {
	      hselect(cimg//ctyp, "BIASCOMB", "yes") | scan (ccom)
	   } else {
	      hselect(cimg//ctyp, "DARKCOMB", "yes") | scan (ccom)
	   }
	   zcomb48 (ccom, imlis, nimg, imgtyp, imgsec, drktim, rdnoise,
			gain, smax, maxn, logfile, comblog, verbose)

	   if ( remove && zcomb.ok ) {
	      if (verbose) {
		 print ("CCD48PROC Removing input images:")
		 print ("    IMDELETE @"//imlis//" ...")
	      }
	      imdelete ("@"//imlis, yes, verify-, default_acti+)
	   }
	}

	if ( anyproc && ( imgtyp == "FLAT" || imgtyp == "SKY" ) ) {
	   # Combine the processed flat/sky frames...
	   if (combine) { ccom = "PERFORM" } else { ccom = "OMIT" }
	   hselect(cimg//ctyp, imgtyp//"COMB", "yes") | scan (ccom)
	   fcomb48 (ccom, imgtyp, imgsec, exptim, rdnoise, gain, smax,
			maxn, cmbr, logfile, comblog, verbose)

	   if ( remove && fcomb48.ok ) {
	      if (verbose) { 
		 print ("CCD48PROC Removing input images:")
		 print ("    IMDELETE @"//imlis//" ...")
	      }
	      imdelete ("@"//imlis, yes, verify-, default_acti+)
	   }
	}


	# Normalize any combined dome or sky flat field calibation frames
	# found in the current directory if the mean pixel value in their
	# image section differs from 1...

	if ( imgtyp == "FLAT" || imgtyp == "SKY" ) {
	   if (normaliz) { ccom = "PERFORM" } else { ccom = "OMIT" }
	   hselect(cimg//ctyp, "normaliz", "yes") | scan (ccom)
	   fnorm48 (ccom, imgtyp, imgsec, logfile, comblog, verbose)
	}


	# Model the scattered light feature due to the field flattener (FLWO
	# 48inch, thinned back-illuminated CCD, Fall 1995). 
	# This operation is not recommended to be used in the same pass in
	# which the flatfields are combined, unless one is certain that
	# there is only 1 combined flat frame per filter.  If not, the
	# following subroutine will crash. (Note that after the Aug 2003
	# overhaul of the 'ccd48' package, only 1 combined frame per filter
	# is produced, unless there were fewer than 3 images to combine to
	# begin with).
	# As the problem with the fieldflattener supposedly is solved, this
	# 'appendix' is likely to be disabled in newer versions of ccd48proc.
	if ( modscat && (imgtyp=="FLAT"||imgtyp=="SKY") ) {
	   # Aug 2003: 'scmod48' is not yet updated. Also at present no
	   #     support for header controled performance of this step.
	   print ("WARNING (modscat=yes): I hope you know what you are doing ...")
	   if ( trmsec != "" ) {
	     rpparse (trmsec, verbose-)
	     scmod48 (imgtyp, (1+rpparse.xhi-rpparse.xlo),
			(1+rpparse.yhi-rpparse.ylo), logfile=logfile,
			verbose=verbose, ok=yes)
	   } else {
	     scmod48 (imgtyp, ncol, nrow, logfile=logfile, verbose=verbose, ok+)
	   }
	}

#	Update task parameter(s)...
	maxcomb = maxn

	print ("\n## CCD48PROC finished.")
	print ("\n## CCD48PROC finished.", >> logfile)

end
