procedure fcomb48 (hdrcom, imgtyp, statsec, expkey, rdnoise, gain, maxval,
		maxcomb, rejtype, logfile, comblog, verbose)

string	hdrcom	=""		{prompt="Header controled command"}
string	imgtyp	=""		{prompt="CCD image data type"}
string	statsec	=""		{prompt="Image section for statistics"}
string	expkey	="exptime"	{prompt="Exposure time header keyword"}
real	rdnoise	=INDEF		{prompt="CCD readout noise (e-)"}
real	gain	=INDEF		{prompt="CCD gain (e-/ADU)"}
real	maxval	=INDEF		{prompt="Maximum non-saturated/linear level"}
int	maxcomb	=7		{prompt="Max # of frames to combine at once"}
string	rejtype ="avsigclip"	{prompt="Type of rejection algorithm",
				 enum="avsigclip|minmax|none"}
string	logfile	=""		{prompt="Text log file"}
string	comblog	=""		{prompt="Text log file image combination"}
bool	verbose	=yes		{prompt="Verbose messages?"}
bool	ok	=no		{prompt="No errors encountered?"}

# @(#) subtask fcomb48            Author: R.A. Jansen -- January 22 1996
# @(#) 
# @(#) subtask called from  'ccd48proc'  to combine  the flat/sky frames
# @(#) after processing.  Parameter 'maxcomb' should match the machine's
# @(#) memory (minus that in use by the X server)  divided by the memory
# @(#) required to store a single image (e.g. (256-64)/4.2 ~45 for 256Mb
# @(#) of RAM and 1024 x 1024 pixel  real valued images).  The 'rdnoise'
# @(#) and 'gain' parameters are at present dummy parameters,  since the
# @(#) images are combined using the  "avsigclip"  rather than "ccdclip"
# @(#) routines, but this may change in the future.
# @(#)
# @(#) Sep 30 2002 -- Headers are now updated  using  'artdata.mkheader'
# @(#)   rather than 'ccdhedit'. Keywords are now '[FLAT|SKY]COMB'  with
# @(#)   value 'PERFORM','OMIT', or 'COMPLETE' and '[FT|SK]CBDATE' (FITS
# @(#)   compliant date + time string). RAJ.
# @(#) Aug 30 2003 -- Added parameters  'hdrcom', 'expkey' and 'maxval';
# @(#)   completely rewrote this task. RAJ.
# @(#) Apr 21 2004 -- Fixed bug that split large image list into unequal
# @(#)   sized batches; fixed bug of badly formatted header entry. Added
# @(#)   check whether input image list contains at least 2 images. RAJ.
# @(#) Mar 15 2004 -- Added parameter 'rejtype'  to allow minmax instead
# @(#)   of avsigclip (default) rejection. RAJ.

begin
	string	cval, clis, dtyp, ctyp, csec, ckey, crej, cnam, csub
	string	root, bstr, cstr, dstr, scle, zrof, wght, cout
	struct	line
	real	ron, eadu, mxval, mnval
	int	nmax, nsub, ntot, i, j, k, m, n
	bool	bok, new

	# Initialize variables...
	cval	= hdrcom
	dtyp	= imgtyp
	csec	= statsec
	ckey	= expkey
	ron	= rdnoise
	eadu	= gain
	mxval	= maxval
	mnval	= INDEF
	if ( mxval != INDEF ) mnval = -0.01*mxval
	nmax	= maxcomb
	crej	= rejtype
	bok	= no
	new	= ( release >= "2.12" )
	ctyp	= "."//envget("imtype")

	# Check type of data and (re)initialize task parameter...
	if ( dtyp != "FLAT" && dtyp != "SKY" ) {
	   print ("ERROR: FCOMB48 called for incorrect image type "//dtyp//"!")
	   ok = no
	   return
	} else {
	   ok = bok
	}

	# Define image combination parameters...
	if ( dtyp == "FLAT" ) {
	   root = "flat"
	   bstr = "FLATCOMB"
	   cstr = "Dome"
	   dstr = "FT"
	}
	if ( imgtyp == "SKY" ) {
	   root = "sky"
	   bstr = "SKYCOMB "
	   cstr = "Sky"
	   dstr = "SK"
	}
	scle = "mean"
	zrof = "none"
	wght = "none"

	# Clean-up any relevant debris...
	if ( access("fcomb.head") ) delete ("fcomb.head", yes, verify-)
	if ( access(dtyp//".subsets") ) delete (dtyp//".subsets", yes, verify-)

	# List the subset lists that are present in current directory...
	dir (dtyp//"_?*.lis", long-, ncols=1, > dtyp//".subsets")

	# Count the number of subset lists to loop over ...
	iwc (dtyp//".subsets")
	nsub = iwc.nwords

	# Open comblog and start logging...
	sysinfo() ; gdate(utc=yes)
	if ( access(comblog) ) {
	   print ("\n## CCD48PROC (FCOMB48)  "//sysinfo.sysmsg//"  "//gdate.sdate//"\n", >> comblog)
	} else {
	   print ("## CCD48PROC (FCOMB48)  "//sysinfo.sysmsg//"  "//gdate.sdate//"\n", >> comblog)
	}

	# Even if 'hdrcom' reads 'COMPLETE', the combined image may have
	# been accidentally deleted. Since combination does not affect the
	# individual images, in this instance, 'COMPLETE' will be ignored.
	if ( cval == "OMIT" ) {
	   if (verbose) print ("    "//bstr//"= \"OMIT\"; no action taken")
	   print ("  "//bstr//"= \"OMIT\"; no action taken", >> logfile)
	   print ("  "//bstr//"= \"OMIT\"; no action taken\n", >> comblog)

	   # Loop over all subset lists and add header keyword 'dtyp'//COMB...
	   gdate(utc=yes)
	   j = 0
	   for ( i = 1 ; i <= nsub ; i = i + 1 ) {
	      rdlist (dtyp//".subsets", i)
	      if (!rdlist.ok) {
		 print ("FCOMB48: ERROR reading line "//str(i)//" from \""//dtyp//".subsets\"!", >> logfile)
		 print ("FCOMB48: ERROR reading line "//str(i)//" from \""//dtyp//".subsets\"!", >> comblog)
		 ok = no
		 return
	      }

	      list = rdlist.cline
	      while ( fscan(list,line) != EOF ) {
		 cnam = line
		 j = j + 1
		 if ( (j%5) == 0 ) gdate(utc=yes)
		 hedit (cnam, "ncombine", "", add-, del+, ver-, show-, upd+)
		 hedit (cnam, dtyp//"COMB", "", add-, del+, ver-, show-, upd+)
		 hedit (cnam, dstr//"CBDATE", "", add-, del+, ver-, show-, upd+)
		 hedit (cnam, "imcmb*", "", add-, del+, ver-, show-, upd+)
		 hedit (cnam, "cc48date", "", add-, del+, ver-, show-, upd+)
		 imheader (cnam, longheader+, > "fcomb.head")
		 sleep(1)
		 printf(bstr//"= %-21s / "//cstr//" frame combination flag\n", "'OMIT'", >> "fcomb.head")
		 print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fcomb.head")
		 mkheader (cnam, "fcomb.head", append-, verbose-)
		 delete ("fcomb.head", yes, verify-)
	      }
	      list = ""

	      flprca
	      sleep(1)
	   }

	} else {
	   # Loop over all subset lists...
	   gdate(utc=yes)
	   for ( i = 1 ; i <= nsub ; i = i + 1 ) {
	      rdlist (dtyp//".subsets", i)
	      if (!rdlist.ok) {
		 print ("FCOMB48: ERROR reading line "//str(i)//" from \""//dtyp//".subsets\"!", >> logfile)
		 print ("FCOMB48: ERROR reading line "//str(i)//" from \""//dtyp//".subsets\"!", >> comblog)
		 ok = no
		 return
	      }
	      clis = rdlist.cline

	      if (verbose) print ("CCD48PROC Input=\"@"//clis//"\" [reject=\""//crej//"\"]:")
	      print ("\nCCD48PROC Input=\"@"//clis//"\" [reject=\""//crej//"\"]:", >> logfile)
	      print ("\nCCD48PROC Input=\"@"//clis//"\" [reject=\""//crej//"\"]:\n", >> comblog)

	      # Count the number of images in current subset list...
	      iwc (clis)
	      ntot = iwc.nwords

	      # Read name or id of current subset...
	      cparse (clis, delim=".", nsel=1, lignore+, mignore+, verbose-)
	      cparse (cparse.cfield, delim="_", nsel=2, ligno+, migno+, verb-)
	      csub = cparse.cfield

	      # Check whether we can combine all images at once or need to
	      # combine them in batches...
	      if ( ntot < nmax ) {
		 # check that there are at least 2 images to combine...
		 if ( ntot < 1 ) {
		    if (verbose) print ("    "//dtyp//"COMB: \""//clis//"\" is empty; no action taken")
		    print ("  "//dtyp//"COMB: \""//clis//"\" is empty; no action taken", >> logfile)
		    print ("  "//dtyp//"COMB: \""//clis//"\" is empty; no action taken\n", >> comblog)
		    ok = no
		    next
		 } else if ( ntot < 2 ) {
		    if (verbose) print ("    "//dtyp//"COMB: only 1 image in \""//clis//"\" -> \""//dtyp//csub//ctyp//"\"")
		    print ("  "//dtyp//"COMB: "//gdate.fdate//" Only 1 image in \""//clis//"\" -> \""//dtyp//csub//ctyp//"\"", >> logfile)
		    print ("  "//dtyp//"COMB: "//gdate.fdate//" Only 1 image in \""//clis//"\" -> \""//dtyp//csub//ctyp//"\"\n", >> comblog)
		    rdlist (clis, 1)
		    imcopy (rdlist.cline, dtyp//csub//ctyp, verbose-)
		    ok = yes
		    next
		 }

		 # Construct the output image name...
		 cout = dtyp//csub//ctyp

		 # Check whether the output frame exists already...
		 chkimg (cout, "noaccess", imtype="", verbose+)
		 if (!chkimg.ok) { return }
		 cout = chkimg.impath//chkimg.imroot//chkimg.imtype

		 if (verbose) print ("    "//bstr//"...  result=\""//cout//"\"")

		 # Combine all images in the input list...
		 if ( new ) {
		    imcombine ("@"//clis, output=cout, headers="", bpmasks="",
			rejmasks="", nrejmasks="", expmasks="", sigma="",
			logfile=comblog, combine="average", reject=crej,
			project=no, outtype="real", outlimits="",
			offsets="none", masktype="none", maskvalue=0., blank=0.,
			scale=scle, zero=zrof, weight=wght, statsec=csec,
			expname=ckey, lthreshold=mnval, hthreshold=mxval,
			nlow=0, nhigh=1, nkeep=1, mclip=yes, lsigma=3.,
			hsigma=3., rdnoise=str(ron), gain=str(eadu),
			snoise="0.", sigscale=0.1, pclip=-0.5, grow=0.5)
		 } else {
		    imcombine ("@"//clis, output=cout, rejmask="", plfile="",
			sigma="", logfile=comblog, combine="average",
			reject=crej, project=no, outtype="real",
			offsets="none", masktype="none", maskvalue=0., blank=0.,
			scale=scle, zero=zrof, weight=wght, statsec=csec,
			expname=ckey, lthreshold=mnval, hthreshold=mxval,
			nlow=0, nhigh=1, nkeep=1, mclip=yes, lsigma=3.,
			hsigma=3., rdnoise=str(ron), gain=str(eadu),
			snoise="0.", sigscale=0.1, pclip=-0.5, grow=0.5)
		 }

		 # Add keywords [IMGTYP]COMB and [FT|SK]CBDATE to image header...
		 gdate(utc=yes)
		 hedit (cout, "ncombine", "", add-, del+, ver-, show-, upd+)
		 hedit (cout, dtyp//"COMB", "", add-, del+, ver-, show-, upd+)
		 hedit (cout, dstr//"CBDATE", "", add-, del+, ver-, show-, upd+)
		 hedit (cout, "imcmb*", "", add-, del+, ver-, show-, upd+)
		 hedit (cout, "cc48date", "", add-, del+, ver-, show-, upd+)
		 imheader (cout, longheader+, > "fcomb.head")
		 sleep(1)
		 printf(bstr//"= %-21s / "//cstr//" frame combination flag\n", "'COMPLETE'", >> "fcomb.head")
		 printf("NCOMBINE= %20d  / Number of "//root//" frames combined\n", ntot, >> "fcomb.head")
		 print (dstr//"CBDATE= '"//gdate.fdate//"' / Combination of "//root//" frames @"//clis//" done", >> "fcomb.head")
		 print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fcomb.head")
		 mkheader (cout, "fcomb.head", append-, verbose-)
		 delete ("fcomb.head", yes, verify-)

		 # Log action to text log files...
		 print ("  "//dtyp//"COMB: "//gdate.fdate//" Combined "//dtyp//" frames (all at once) -> \""//cout//"\"", >> logfile)
		 print ("  "//dtyp//"COMB: "//gdate.fdate//" Combined "//dtyp//" frames (all at once) -> \""//cout//"\"\n", >> comblog)

		 # Add header keyword 'dtyp'//COMB to images in current list...
		 list = clis
		 while ( fscan(list,line) != EOF ) {
		    cnam = line
		    hedit (cnam, "ncombine", "", add-, del+, ver-, show-, upd+)
		    hedit (cnam, dtyp//"COMB", "", add-, del+, ver-, show-, upd+)
		    hedit (cnam, dstr//"CBDATE", "", add-, del+, ver-, show-, upd+)
		    hedit (cnam, "imcmb*", "", add-, del+, ver-, show-, upd+)
		    hedit (cnam, "cc48date", "", add-, del+, ver-, show-, upd+)
		    imheader (cnam, longheader+, > "fcomb.head")
		    sleep(1)
		    printf(bstr//"= %-21s / "//cstr//" frame combination flag\n", "'COMPLETE'", >> "fcomb.head")
		    print (dstr//"CBDATE= '"//gdate.fdate//"' / Combination of "//root//" frames @"//clis//" done", >> "fcomb.head")
		    print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fcomb.head")
		    mkheader (cnam, "fcomb.head", append-, verbose-)
		    delete ("fcomb.head", yes, verify-)
		 }
		 list = ""

	      } else {
		 # Number of batches to subdivide image list into...
		 m = nint(1.0*ntot/nmax)

		 # Write the m lists (batches) of images to combine...
		 n = 1
		 for ( j = 1 ; j <= m ; j = j + 1 ) {
		    cnam = "fcomb"//str(j)//".lis"
		    if ( access(cnam) ) delete (cnam, yes, verify-)

		    for ( k = n ; k <= nint(1.0*j*ntot/m) ; k = k + 1 ) {
		       rdlist (clis, k)
		       if (!rdlist.ok) {
			  print ("FCOMB48: ERROR reading line "//str(k)//" from \""//clis//"\"!", >> logfile)
			  print ("FCOMB48: ERROR reading line "//str(k)//" from \""//clis//"\"!")
			  ok = no
			  return
		       } else {
			  print (rdlist.cline, >> cnam)
		       }
		    }

		    n = nint(1.0*j*ntot/m) + 1
		 }

		 # Now combine the images within each of the batches...
		 if ( access("fcomb.lis") ) delete ("fcomb.lis", yes, verify-)
		 for ( j = 1 ; j <= m ; j = j + 1 ) {
		    cout = dtyp//csub//str(j)//ctyp
		    print (cout, >> "fcomb.lis")
		    cnam = "fcomb"//str(j)//".lis"
		    iwc (cnam)

		    # Check whether the output frame exists already...
		    chkimg (cout, "noaccess", imtype="", verbose+)
		    if (!chkimg.ok) { return }
		    cout = chkimg.impath//chkimg.imroot//chkimg.imtype

		    if (verbose) printf("    "//bstr//"... batch %d -> \""//cout//"\" (NCOMBINE=%d)\n", j, iwc.nlines)

		    if ( new ) {
			imcombine ("@"//cnam, output=cout, headers="", bpmasks="",
				rejmasks="", nrejmasks="", expmasks="", sigma="",
				logfile=comblog, combine="average", reject=crej,
				project=no, outtype="real", outlimits="",
				offsets="none", masktype="none", maskvalue=0., blank=0.,
				scale=scle, zero=zrof, weight=wght, statsec=csec,
				expname=ckey, lthreshold=mnval, hthreshold=mxval,
				nlow=0, nhigh=1, nkeep=1, mclip=yes, lsigma=3.,
				hsigma=3., rdnoise=str(ron), gain=str(eadu),
				snoise="0.", sigscale=0.1, pclip=-0.5, grow=0.5)
		    } else {
			imcombine ("@"//cnam, output=cout, rejmask="", plfile="",
				sigma="", logfile=comblog, combine="average",
				reject=crej, project=no, outtype="real",
				offsets="none", masktype="none", maskvalue=0., blank=0.,
				scale=scle, zero=zrof, weight=wght, statsec=csec,
				expname=ckey, lthreshold=mnval, hthreshold=mxval,
				nlow=0, nhigh=1, nkeep=1, mclip=yes, lsigma=3.,
				hsigma=3., rdnoise=str(ron), gain=str(eadu),
				snoise="0.", sigscale=0.1, pclip=-0.5, grow=0.5)
		    }

		    # Add keywords [IMGTYP]COMB and [FT|SK]CBDATE to image header...
		    gdate(utc=yes)
		    hedit (cout, "ncombine", "", add-, del+, ver-, show-, upd+)
		    hedit (cout, dtyp//"COMB", "", add-, del+, ver-, show-, upd+)
		    hedit (cout, dstr//"CBDATE", "", add-, del+, ver-, show-, upd+)
		    hedit (cout, "imcmb*", "", add-, del+, ver-, show-, upd+)
		    hedit (cout, "cc48date", "", add-, del+, ver-, show-, upd+)
		    imheader (cout, longheader+, > "fcomb.head")
		    sleep(1)
		    printf(bstr//"= %-21s / "//cstr//" frame combination flag\n", "'COMPLETE'", >> "fcomb.head")
		    printf("NCOMBINE= %20d  / Number of "//root//" frames combined\n", iwc.nlines, >> "fcomb.head")
		    print (dstr//"CBDATE= '"//gdate.fdate//"' / Combination of "//root//" frames @"//cnam//" done", >> "fcomb.head")
		    print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fcomb.head")
		    mkheader (cout, "fcomb.head", append-, verbose-)
		    delete ("fcomb.head", yes, verify-)

		    # Log action to text log files...
		    if ( j == 1 ) {
		       printf("  "//dtyp//"COMB: "//gdate.fdate//" Combined "//dtyp//" frames (batch %d) -> \""//cout//"\"\n\n", j, >> comblog)
		    } else {
		       printf("\n  "//dtyp//"COMB: "//gdate.fdate//" Combined "//dtyp//" frames (batch %d) -> \""//cout//"\"\n\n", j, >> comblog)
		    }
		    printf("  "//dtyp//"COMB: "//gdate.fdate//" Combined "//dtyp//" frames (batch %d) -> \""//cout//"\"\n", j, >> logfile)

		 }

		 # Finally, combine the frames combined per batch in a single
		 # output frame...
		 cout = dtyp//csub//ctyp

		 # Check whether the output frame exists already...
		 chkimg (cout, "noaccess", imtype="", verbose+)
		 if (!chkimg.ok) { return }
		 cout = chkimg.impath//chkimg.imroot//chkimg.imtype

		 if (verbose) {
		    printf("    "//bstr//"... batches 1--%d -> \""//cout//"\" (NCOMBINE=%d)\n", m, ntot)
		 }
		 if ( new ) {
		    imcombine ("@fcomb.lis", output=cout, headers="", bpmasks="",
			rejmasks="", nrejmasks="", expmasks="", sigma="",
			logfile=comblog, combine="average", reject=crej,
			project=no, outtype="real", outlimits="",
			offsets="none", masktype="none", maskvalue=0., blank=0.,
			scale=scle, zero=zrof, weight=wght, statsec=csec,
			expname=ckey, lthreshold=mnval, hthreshold=mxval,
			nlow=0, nhigh=1, nkeep=1, mclip=yes, lsigma=3.,
			hsigma=3., rdnoise=str(ron), gain=str(eadu),
			snoise="0.", sigscale=0.1, pclip=-0.5, grow=0.5)
		 } else {
		    imcombine ("@fcomb.lis", output=cout, rejmask="", plfile="",
			sigma="", logfile=comblog, combine="average",
			reject=crej, project=no, outtype="real",
			offsets="none", masktype="none", maskvalue=0., blank=0.,
			scale=scle, zero=zrof, weight=wght, statsec=csec,
			expname=ckey, lthreshold=mnval, hthreshold=mxval,
			nlow=0, nhigh=1, nkeep=1, mclip=yes, lsigma=3.,
			hsigma=3., rdnoise=str(ron), gain=str(eadu),
			snoise="0.", sigscale=0.1, pclip=-0.5, grow=0.5)
		 }

		 # Add keywords [IMGTYP]COMB and [FT|SK]CBDATE to image header...
		 gdate(utc=yes)
		 hedit (cout, "ncombine", "", add-, del+, ver-, show-, upd+)
		 hedit (cout, dtyp//"COMB", "", add-, del+, ver-, show-, upd+)
		 hedit (cout, dstr//"CBDATE", "", add-, del+, ver-, show-, upd+)
		 hedit (cout, "imcmb*", "", add-, del+, ver-, show-, upd+)
		 hedit (cout, "cc48date", "", add-, del+, ver-, show-, upd+)
		 imheader (cout, longheader+, > "fcomb.head")
		 sleep(1)
		 printf(bstr//"= %-21s / "//cstr//" frame combination flag\n", "'COMPLETE'", >> "fcomb.head")
		 printf("NCOMBINE= %20d  / Number of "//root//" frames combined\n", ntot, >> "fcomb.head")
		 print (dstr//"CBDATE= '"//gdate.fdate//"' / Combination of "//root//" frames @"//clis//" done", >> "fcomb.head")
		 print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fcomb.head")
		 mkheader (cout, "fcomb.head", append-, verbose-)
		 delete ("fcomb.head,fcomb.lis", yes, verify-)

		 # Log action to text log files...
		 printf("  "//dtyp//"COMB: "//gdate.fdate//" Combined batches 1--%d -> \""//cout//"\"\n", m, >> logfile)
		 printf("\n  "//dtyp//"COMB: "//gdate.fdate//" Combined batches 1--%d -> \""//cout//"\"\n\n", m, >> comblog)

		 # Add header keyword 'dtyp'//COMB to images in current list...
		 list = clis
		 while ( fscan(list,line) != EOF ) {
		    cnam = line
		    hedit (cnam, "ncombine", "", add-, del+, ver-, show-, upd+)
		    hedit (cnam, dtyp//"COMB", "", add-, del+, ver-, show-, upd+)
		    hedit (cnam, dstr//"CBDATE", "", add-, del+, ver-, show-, upd+)
		    hedit (cnam, "imcmb*", "", add-, del+, ver-, show-, upd+)
		    hedit (cnam, "cc48date", "", add-, del+, ver-, show-, upd+)
		    imheader (cnam, longheader+, > "fcomb.head")
		    sleep(1)
		    printf(bstr//"= %-21s / "//cstr//" frame combination flag\n", "'COMPLETE'", >> "fcomb.head")
		    print (dstr//"CBDATE= '"//gdate.fdate//"' / Combination of "//root//" frames @"//clis//" done", >> "fcomb.head")
		    print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fcomb.head")
		    mkheader (cnam, "fcomb.head", append-, verbose-)
		    delete ("fcomb.head", yes, verify-)
		 }
		 list = ""
	      }

	      flprca
	      sleep(1)
	   }

	}

	# Clean-up...
	delete ("fcomb?.lis", yes, verify-)
	delete (dtyp//".subsets", yes, verify-)

	# Update task parameter...
	ok = yes

end
