procedure fnorm48 (hdrcom, imgtyp, statsec, logfile, comblog, verbose)

string	hdrcom	=""		{prompt="Header controled command"}
string	imgtyp	=""		{prompt="CCD image data type"}
string	statsec	=""		{prompt="Image section for statistics"}
string	logfile	=""		{prompt="Text log file"}
string	comblog	=""		{prompt="Text log file image combination"}
bool	verbose	=yes		{prompt="Verbose messages?"}
bool	ok	=no		{prompt="No errors encountered?"}

# @(#) subtask fnorm48             Author: R.A. Jansen -- August 30 2003
# @(#) 
# @(#) subtask called from  'ccd48proc'  to normalize the combined dome/
# @(#) sky flat fields if their mean pixel value differs from 1.0. After
# @(#) normalization,  three keywords are written  to the image headers:
# @(#) 'NORMALIZ' (valued 'PERFORM', 'OMIT', or 'COMPLETE'),  'NMLZMEAN'
# @(#) listing the mean pixel value before normalization, and 'NMLZDATE'
# @(#) (a FITS Y2K compliant date+time string).
# @(#)
# @(#) Jan 22 1996 -- Wrote subtask 'fcomb48',  of which this task was a
# @(#)                part until the v2.12 overhaul, dd. Aug 2003.
# @(#) Aug 30 2003 -- Split normalization part from 'fcomb48' to be able
# @(#)                to normalize flat fields previously combined, even
# @(#)                when 'anyproc=no'. Complete rewrite of task. RAJ

begin
	string	cval, dtyp, csec, ctyp, root, cstr, cnam, clis, csub, cfil
	struct	line
	real	rmean
	int	nsubs, i, j, k, m, n
	bool	bok

	# Initialize variables...
	cval	= hdrcom
	dtyp	= imgtyp
	csec	= statsec
	bok	= no
	ctyp	= "."//envget("imtype")
	rmean	= INDEF


	if (verbose) printf("\n")

	# Check type of data and (re)initialize task parameter...
	if ( dtyp != "FLAT" && dtyp != "SKY" ) {
	   print ("ERROR: FNORM48 called for incorrect image type "//dtyp//"!")
	   ok = no
	   return
	} else {
	   ok = bok
	}

	# Define image data type strings...
	if ( dtyp == "FLAT" ) {
	   root = "flat"
	   cstr = "Dome"
	}
	if ( imgtyp == "SKY" ) {
	   root = "sky"
	   cstr = "Sky"
	}

	# Clean-up any relevant debris...
	if ( access("fnorm.head") ) delete ("fnorm.head", yes, verify-)
	if ( access(dtyp//".subsets") ) delete (dtyp//".subsets", yes, verify-)

	# List the subset lists that are present in current directory...
	dir (dtyp//"?*.lis", long-, ncols=1, > dtyp//".subsets")

	# Count the number of subset lists to loop over ...
	iwc (dtyp//".subsets")
	nsubs = iwc.nwords

	# Open comblog and start logging...
	sysinfo() ; gdate(utc=yes)
	if ( access(comblog) ) {
	   print ("\n## CCD48PROC (FNORM48)  "//sysinfo.sysmsg//"  "//gdate.sdate//"\n", >> comblog)
	} else {
	   print ("## CCD48PROC (FNORM48)  "//sysinfo.sysmsg//"  "//gdate.sdate//"\n", >> comblog)
	}

	# Even if 'hdrcom' reads 'COMPLETE', the normalized image may have
	# been accidentally rescaled. Since normalization does not affect the
	# individual original flat/sky images, in this instance, 'COMPLETE'
	# will be ignored.
	if ( cval == "OMIT" ) {
	   if (verbose) print ("    NORMALIZ= \"OMIT\"; no action taken")
	   print ("\n  NORMALIZ= \"OMIT\"; no action taken", >> logfile)
	   print ("\n  NORMALIZ= \"OMIT\"; no action taken\n", >> comblog)

	   # Loop over all subset lists and add header keyword NORMALIZ...
	   gdate(utc=yes)
	   j = 0
	   for ( i = 1 ; i <= nsubs ; i = i + 1 ) {
	      rdlist (dtyp//".subsets", i)
	      if (!rdlist.ok) {
		 print ("FNORM48: ERROR reading line "//str(i)//" from \""//dtyp//".subsets\"!", >> logfile)
		 print ("FNORM48: ERROR reading line "//str(i)//" from \""//dtyp//".subsets\"!", >> comblog)
		 ok = no
		 return
	      }

	      list = rdlist.cline
	      while ( fscan(list,line) != EOF ) {
		 cnam = line
		 j = j + 1
		 if ( (j%5) == 0 ) gdate(utc=yes)
		 hedit (cnam, "normaliz", "", add-, del+, ver-, show-, upd+)
		 hedit (cnam, "nmlzmean", "", add-, del+, ver-, show-, upd+)
		 hedit (cnam, "nmlzdate", "", add-, del+, ver-, show-, upd+)
		 hedit (cnam, "cc48date", "", add-, del+, ver-, show-, upd+)
		 imheader (cnam, longheader+, > "fnorm.head")
		 printf("NORMALIZ= %-21s / "//cstr//" flat normalization flag\n", "'OMIT'", >> "fnorm.head")
		 print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fnorm.head")
		 mkheader (cnam, "fnorm.head", append-, verbose-)
		 delete ("fnorm.head", yes, verify-)
	      }
	      list = ""
	   }

	} else {
	   printf("\n", >> logfile)
	   printf("\n", >> comblog)

	   # Loop over all subset lists...
	   gdate(utc=yes)
	   for ( i = 1 ; i <= nsubs ; i = i + 1 ) {
	      rdlist (dtyp//".subsets", i)
	      if (!rdlist.ok) {
		 print ("FNORM48: ERROR reading line "//str(i)//" from \""//dtyp//".subsets\"!", >> logfile)
		 print ("FNORM48: ERROR reading line "//str(i)//" from \""//dtyp//".subsets\"!", >> comblog)
		 ok = no
		 return
	      }
	      clis = rdlist.cline

	      # Read name or id of current subset...
	      cparse (clis, delim=".", nsel=1, lignore+, mignore+, verbose-)
	      cparse (cparse.cfield, delim="_", nsel=2, ligno+, migno+, verb-)
	      csub = cparse.cfield
	      cfil = dtyp//csub//ctyp

	      # Check that a flat field for this subset exists...
	      chkimg (cfil, "access", imtype="", verbose+)
	      if (!chkimg.ok) {
		 print("WARNING: frame \""//cfil//"\" not found! Continuing...")
		 next
	      }
	      cfil = chkimg.impath//chkimg.imroot//chkimg.imtype

	      if (verbose) printf("    NORMALIZ "//dtyp//csub//ctyp//" ...")

	      # Determine mean pixel value in statistics section...
	      imstat (cfil//csec, fields="mean", lower=INDEF, upper=INDEF,
			nclip=3., lsigma=4., usigma=4., binwidth=0.1,
			format-, cache-) | scan(rmean)
	      if ( rmean == INDEF ) {
		 print ("FNORM48: ERROR determining mean pixel value of \""//cfil//csec//"\"!", >> logfile)
		 print ("FNORM48: ERROR determining mean pixel value of \""//cfil//csec//"\"!\n", >> comblog)
		 ok = no
		 return
	      }

	      # Normalize flat field for current subset...
	      imarith (cfil, "/", rmean, cfil, title=dtyp//csub, divzero=0.,
			hparams="", pixtype="real", calctype="real",
			verbose-, noact-)

	      # Add keywords NORMALIZ, NMLZMEAN, and NMLZDATE to image header...
	      gdate(utc=yes)
	      hedit (cfil, "normaliz", "", add-, del+, ver-, show-, upd+)
	      hedit (cfil, "nmlzmean", "", add-, del+, ver-, show-, upd+)
	      hedit (cfil, "nmlzdate", "", add-, del+, ver-, show-, upd+)
	      hedit (cfil, "cc48date", "", add-, del+, ver-, show-, upd+)
	      imheader (cfil, longheader+, > "fnorm.head")
	      printf("NORMALIZ= %-21s / "//cstr//" flat normalization flag\n", "'COMPLETE'", >> "fnorm.head")
	      printf("NMLZMEAN= %20.2f  / Mean level (ADU) prior to normalization\n", rmean, >> "fnorm.head")
	      print ("NMLZDATE= '"//gdate.fdate//"' / "//cstr//" flat normalization done", >> "fnorm.head")
	      print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fnorm.head")
	      mkheader (cfil, "fnorm.head", append-, verbose-)
	      delete ("fnorm.head", yes, verify-)

	      if (verbose) printf("  done (mean=%8f ADU)\n", rmean)

	      # Log action to text log files...
	      print (cfil//":", >> comblog)
	      printf("  NORMALIZ:  "//gdate.fdate//" Normalized "//cstr//" flat field: mean=%8f\n", rmean, >> comblog)
	      printf("  NORMALIZ:  "//gdate.fdate//" Normalized "//cstr//" flat field \""//cfil//"\": mean=%8f\n", rmean, >> logfile)

	      # Add header keyword NORMALIZ to images in current subset list...
	      list = clis
	      j = 0
	      while ( fscan(list,line) != EOF ) {
		 cnam = line
		 j = j + 1
		 if ( (j%5) == 0 ) gdate(utc=yes)
		 hedit (cnam, "normaliz", "", add-, del+, ver-, show-, upd+)
		 hedit (cnam, "nmlzmean", "", add-, del+, ver-, show-, upd+)
		 hedit (cnam, "nmlzdate", "", add-, del+, ver-, show-, upd+)
		 hedit (cnam, "cc48date", "", add-, del+, ver-, show-, upd+)
		 imheader (cnam, longheader+, > "fnorm.head")
		 printf("NORMALIZ= %-21s / "//cstr//" flat normalization flag\n", "'COMPLETE'", >> "fnorm.head")
		 print ("NMLZDATE= '"//gdate.fdate//"' / "//cstr//" flat normalization done", >> "fnorm.head")
		 print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fnorm.head")
		 mkheader (cnam, "fnorm.head", append-, verbose-)
		 delete ("fnorm.head", yes, verify-)
	      }
	      list = ""
	   }

	}

	# Clean-up...
	delete (dtyp//".subsets", yes, verify-)

	# Update task parameter...
	ok = yes

end
