procedure fresp48 (hdrcom, image, forder, surface, logfile, resplog,
		verbose, first)

string	hdrcom	=""		{prompt="Header controled command"}
string	image	=""		{prompt="Image name"}
int	forder	=INDEF		{prompt="Fitting order"}
bool	surface	=yes		{prompt="Fit 2-D surface i.s.o. 1-D lines?"}
string	logfile	=""		{prompt="Text logfile ccd48proc"}
string	resplog	=""		{prompt="Text logfile response fit"}
bool	verbose	=yes		{prompt="Verbose messages?"}
bool	first	=no		{prompt="First image in list?"}

# @(#) subtask fresp48          Author: R.A. Jansen -- September 26 2003
# @(#) 
# @(#) subtask called from 'ccd48proc' to fit  a high-order surface to a
# @(#) dome or sky flat image and return the pixel-to-pixel response.
# @(#)
# @(#) Mar 12 2004 -- added 2-D/1-D response fitting switch [RAJ+KTD]
# @(#) Apr 21 2004 -- added 'RSPFTYPE' keyword, to record type and order
# @(#)                of the response fit in the image header [RAJ]

begin
	string	cval, cimg, cstr
 	int	iord

	# Initialize variables...
	cval  = hdrcom
	cimg  = image
	iord  = forder
	cstr  = "fit1d,spline3,order="//str(iord)
	if ( surface ) cstr = "fit2d,spline3,order="//str(iord)
	if ( resplog == "" ) resplog = "fitresp.log"

	# Clean-up any relevant debris...
	if ( access("fresp.head") ) delete("fresp.head", yes, verify-)

	# If first image in list, open resplog and start logging...
	if (first) {
	   sysinfo() ; gdate(utc=yes)
	   if ( access(resplog) ) {
	      print ("\n## CCD48PROC (FRESP48)  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n", >> resplog)
	   } else {
	      print ("## CCD48PROC (FRESP48)  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n", >> resplog)
	   }
	}

	# Check whether image was processed already...
	if ( cval == "COMPLETE" ) {
	   if (verbose) print ("    RESPFIT:  image was processed already; no action taken")
	   print ("  RESPFIT = \"COMPLETE\"; no action taken", >> logfile)
	   print (cimg//": RESPFIT = \"COMPLETE\"; no action taken", >> resplog)

	} else if ( cval == "OMIT" ) {
	   if (verbose) print ("    RESPFIT = \"OMIT\"; no action taken")
	   print ("  RESPFIT = \"OMIT\"; no action taken", >> logfile)
	   print (cimg//": RESPFIT = \"OMIT\"; no action taken", >> resplog)

	   # Add header keyword RESPFIT...
	   gdate(utc=yes)
	   hedit (cimg, "respfit", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "fresp.head")
	   printf("RESPFIT = %-21s / Pixel-to-pixel response fit flag\n", "'OMIT'", >> "fresp.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fresp.head")
	   mkheader (cimg, "fresp.head", append-, verbose-)
	   delete ("fresp.head", yes, verify-)

	} else {
	   if (surface) {
	      if (verbose) printf("    RESPFIT (fit2D,order=%d) ...", iord)

	      # Fit out the low-order variations and return a pixel-to-pixel
	      # response frame...
	      if ( access("tmpfresp.fits") ) delete ("tmpfresp.fits", yes, verify-)
	      imsurfit (cimg, "tmpfresp.fits", xorder=iord, yorder=iord,
		type_output="response", function="spline3", cross_terms=yes,
		xmedian=5, ymedian=5, median_perce=50., lower=2., upper=3.,
		ngrow=10, niter=5, regions="all", rows="*", columns="*",
		border="", sections="", circle="", div_min=INDEF)

	      imdelete (cimg, yes, verify-, default_acti+)
	      imrename ("tmpfresp.fits", cimg, verbose-)

	   } else {
	      if (verbose) printf("    RESPFIT (fit1D,order=%d) ...", iord)

	      # Fit out the low-order variations and return a pixel-to-pixel
	      # response frame...
	      if ( access("tmpfresp1.fits") ) delete ("tmpfresp1.fits", yes, verify-)
	      fit1d (cimg, "tmpfresp1.fits", type="ratio", axis=1,
		interactive=no, sample="*", naverage=-5, function="spline3",
		order=iord, low_reject=2., high_reject=3., niterate=3, grow=10.,
		graphics="stdgraph", cursor="")
	      if ( access("tmpfresp2.fits") ) delete ("tmpfresp2.fits", yes, verify-)
	      fit1d ("tmpfresp1.fits", "tmpfresp2.fits", type="ratio", axis=2,
		interactive=no, sample="*", naverage=-5, function="spline3",
		order=iord, low_reject=2., high_reject=3., niterate=3, grow=10.,
		graphics="stdgraph", cursor="")
	      if ( access("tmpfresp3.fits") ) delete ("tmpfresp3.fits", yes, verify-)
	      fit1d ("tmpfresp2.fits", "tmpfresp3.fits", type="ratio", axis=1,
		interactive=no, sample="*", naverage=-5, function="spline3",
		order=iord, low_reject=2., high_reject=3., niterate=3, grow=10.,
		graphics="stdgraph", cursor="")

	      imdelete (cimg, yes, verify-, default_acti+)
	      imrename ("tmpfresp3.fits", cimg, verbose-)
	      delete ("tmpfresp?.fits", yes, verify=no)

	   }

	   # Add keywords RESPFIT, RSPFTYPE and RSPFDATE...
	   gdate(utc=yes)
	   hedit (cimg, "respfit", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "rspftype", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "rspfdate", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "fresp.head")
	   printf("RESPFIT = %-21s / Pixel-to-pixel response fit flag\n", "'COMPLETE'", >> "fresp.head")
	   printf("RSPFTYPE= %-"//strlen(cstr)//"s / Type and order of fit\n", "'"//cstr//"'", >> "fresp.head")
	   print ("RSPFDATE= '"//gdate.fdate//"' / Pixel-to-pixel response fit done", >> "fresp.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fresp.head")
	   mkheader (cimg, "fresp.head", append-, verbose-)
	   delete ("fresp.head", yes, verify-)

	   if (verbose) printf("  done\n")

	   # Log action to text log files...
	   print (cimg//":", >> resplog)
	   print ("  RESPFIT:  "//gdate.fdate//" Pixel-to-pixel response fit ("//cstr//")", >> resplog)
	   print ("  RESPFIT:  "//gdate.fdate//" Pixel-to-pixel response fit ("//cstr//")", >> logfile)

	}

end
