procedure fstar48 (image,fwhm,rdnoise,gain,secpix,logfile,radplots)

string	image		{prompt="Image name"}
real	fwhm		{prompt="FWHM of stellar PSF (estimate)"}
real	rdnoise		{prompt="CCD read out noise (e-)"}
real	gain		{prompt="CCD gain (e-/ADU)"}
real	secpix		{prompt="CCD pixel scale (arcsec/pix)"}
string	logfile		{prompt="Text logfile ccd48proc"}
bool	radplots=no	{prompt="Output radial profile plots?"}

# @(#) subtask fstar48            Author: R.A. Jansen -- January 23 1996
# @(#) 
# @(#) subtask called from 'ccd48proc' to automatically find all stellar
# @(#) images in the input frame and output the pixel coordinates of the
# @(#) stellar images to file.
# @(#)

begin
	string	name, filt, dstr, logstr, coorfile, magfile, plfile
        real	rmode, expos, noise, sigma, scale
        struct	line
        int	imglen, nl

        name = image
        scale = secpix

	unlearn chkimg ; chkimg (name, "access", imtype="", verbose=no)
        imglen = chkimg.imglen

        if ( access("tmpcp.findstar") ) { delete ("tmpcp.findstar") }

        hedit (name, "findstar", ".", add=no, delete=no, verify=yes, show=yes,
               update=yes, > "tmpcp.findstar")

        nl = 0
        list = "tmpcp.findstar"
        while ( fscan(list,line) != EOF ) { nl += 1 }        

        if ( nl > 0 ) {
           print("FINDSTAR: "//name//" has been processed already; skipping image", >> logfile)
           print("    FINDSTAR: "//name//" has been processed already; skipping image")
        }
        else {
#          Test whether appropriate packages/tasks are loaded
           if ( !defpac("digiphot") ) {
              print("ERROR: Package digiphot not loaded!")
              return
           }
           if ( !defpac("apphot") ) {
              print("ERROR: Package digiphot.apphot not loaded!")
              return
           }

           unlearn apphot
   
           imgets (name, "exptime"); expos = real(imgets.value)
           imgets (name, "filter"); filt = imgets.value

#          Calculate estimate for the sigma in the sky background
           imgstat (name, domode=yes)
   	   rmode = imgstat.rmode
           noise = ((rdnoise*rdnoise)/gain)/gain
           sigma = int( 1000.*sqrt(rmode+noise) )/1000.
   
#          Write out data dependent parameter file "dataparms.par"
           if ( access("dataparms.par") ) { delete ("dataparms.par") }
           print("scale,r,h,",scale,",0.,,\"Image scale in units per pixel\"", >> "dataparms.par")
           print("fwhmpsf,r,h,",fwhm,",0.,,\"FWHM of the PSF in scale units\"", >> "dataparms.par")
           print("emission,b,h,yes,,,\"Features are positive ?\"", >> "dataparms.par")
           print("sigma,r,h,",sigma,",,,\"Standard deviation of background in counts\"", >> "dataparms.par")
           print("datamin,r,h,-100.,,,\"Minimum good data value\"", >> "dataparms.par")
           print("datamax,r,h,32000.,,,\"Maximum good data value\"", >> "dataparms.par")
           print("noise,s,h,\"poisson\",|constant|poisson|,,\"Noise model\"", >> "dataparms.par")
           print("ccdread,s,h,\"rdnoise\",,,\"CCD readout noise image header keyword\"", >> "dataparms.par")
           print("gain,s,h,\"gain\",,,\"CCD gain image header keyword\"", >> "dataparms.par")
           print("readnoise,r,h,",rdnoise,",,,\"CCD readout noise in electrons\"", >> "dataparms.par")
           print("epadu,r,h,",gain,",,,\"Gain in electrons per count\"", >> "dataparms.par")
           print("exposure,s,h,\"exptime\",,,\"Exposure time image header keyword\"", >> "dataparms.par")
           print("airmass,s,h,\"airmass\",,,\"Airmass image header keyword\"", >> "dataparms.par")
           print("filter,s,h,\"filter\",,,\"Filter image header keyword\"", >> "dataparms.par")
           print("obstime,s,h,\"utopen\",,,\"Time of observation image header keyword\"", >> "dataparms.par")
           print("itime,r,h,",expos,",,,\"Exposure time\"", >> "dataparms.par")
           print("xairmass,r,h,INDEF,,,\"Airmass\"", >> "dataparms.par")
           print("ifilter,s,h,\"",filt,"\",,,\"Filter\"", >> "dataparms.par")
           print("otime,s,h,INDEF,,,\"Time of observation\"", >> "dataparms.par")
           print("mode,s,h,\"ql\",,,", >> "dataparms.par")
   
#          Write out star detection parameter file "apphot.findpars"
           if ( access("findparms.par") ) { delete ("findparms.par") }
           print("threshold,r,h,4.,,,\"Threshold in sigma for feature detection\"", >> "findparms.par")
           print("nsigma,r,h,1.5,,,\"Width of convolution kernel in sigma\"", >> "findparms.par")
           print("ratio,r,h,1.,0.,1.,\"Ratio of minor to major axis of Gaussian kernel\"", >> "findparms.par")
           print("theta,r,h,0.,0.,180.,\"Position angle of major axis of Gaussian kernel\"", >> "findparms.par")
           print("sharplo,r,h,0.2,,,\"Lower bound on sharpness for feature detection \"", >> "findparms.par")
           print("sharphi,r,h,1.,,,\"Upper bound on sharpness for  feature detection\"", >> "findparms.par")
           print("roundlo,r,h,-1.,,,\"Lower bound on roundness for feature detection\"", >> "findparms.par")
           print("roundhi,r,h,1.,,,\"Upper bound on roundness for feature detection\"", >> "findparms.par")
           print("mkdetections,b,h,no,,,\"Mark detected stars on the display ?\"", >> "findparms.par")
           print("mode,s,h,\"ql\",,,", >> "findparms.par")
   
#          In case we are working over the network on another machine, reassure
#          IRAF that it really does know how to plot things (...)
           stty xterm

#          Find all stars ...
           if ( access(substr(name,1,imglen)//".coo.1") ) {
              print("!mv "//substr(name,1,imglen)//".coo.1 "//substr(name,1,imglen)//".coo.1.OLD", >> "tmpfstr48.cl")
              cl < tmpfstr48.cl
              delete("tmpfstr48.cl")
           }
           daofind (substr(name,1,imglen), output="default", starmap="", skymap="",
                    datapars="dataparms.par", findpars="findparms.par", boundary="reflect",
                    constant=0., interactive=no, verify=no, update=yes, verbose=no,
                    graphics="stdgraph", display="stdimage", icommands="", gcommands="")
   
#          Add header keyword FINDSTAR and log info to image header
           gdate();  dstr = gdate.date
           print (name//": "//dstr//" Daofind star finding using FWHM="//str(fwhm), >> logfile)
           logstr = dstr//" Daofind star finding using FWHM="//str(fwhm)
           ccdhedit (name, "findstar", logstr, type="string")
   
           coorfile = substr(name,1,imglen)//".coo.1"
           magfile = substr(name,1,imglen)//".mag.1"
           plfile = ""
           if (radplots) { plfile = substr(name,1,imglen)//".gki" }

#          Do quick aperture photometry on stars found ...
           qphot (name, 5., 20., 20., "3.,5.,7.,9.,11.,13.,15.,17.", coords=coorfile,
                  output=magfile, plotfile=plfile, zmag=26., exposure="exptime",
                  airmass="airmass", filter="filter", obstime="utopen", epadu=gain,
                  interactive=no, radplots=no, verbose=no, graphics="stdgraph",
                  display="stdimage", icommands="", gcommands="")

#          Add header keyword STARPHOT and log info to image header
           gdate();  dstr = gdate.date
           print (name//": "//dstr//" Quick stellar photometry with ZMAG=26.0", >> logfile)
           logstr = dstr//" Quick stellar photometry with ZMAG=26.0"
           ccdhedit (name, "starphot", logstr, type="string")

        }   

        delete ("tmpcp.findstar")

end
