procedure ftcor48 (hdrcom, image, flat, subset, logfile, flatlog,
		verbose, first)

string	hdrcom	=""		{prompt="Header controled command"}
string	image	=""		{prompt="Image name"}
string	flat	=""		{prompt="Flatfield (root)name"}
string 	subset	=""		{prompt="Image subset"}
string	logfile	=""		{prompt="Text logfile ccd48proc"}
string	flatlog	=""		{prompt="Text logfile flatfielding"}
bool	verbose	=yes		{prompt="Verbose messages?"}
bool	first	=no		{prompt="First image in list?"}

# @(#) subtask ftcor48            Author: R.A. Jansen -- January 17 1996
# @(#) 
# @(#) subtask called from  'ccd48proc'  to flat field an image (ie., to
# @(#) correct for pixel-to-pixel variations in Q.E. of the detector).
# @(#)
# @(#) Nov  3 2002 -- Headers are now updated  using  'artdata.mkheader'
# @(#)   rather than 'ccdhedit'.  Keywords are now 'FLATCOR' (with value
# @(#)   'PERFORM','OMIT', or 'COMPLETE'), 'FTCRIMAG' with image name as
# @(#)   its value and 'FTCRDATE' (FITS Y2K compliant date+time string).
# @(#) Aug 14 2003 -- Added 'hdrcom' to support header keyword controled
# @(#)   data reduction (a la 'calstis') and added 'verbose' switch. RAJ
# @(#) Apr 21 2004 -- Fixed bug of FTCRIMAG header record >80 chars, and
# @(#)   bug of double subset identifier. Added check that flat frame is
# @(#)   normalized; if not, (re)normalize. RAJ

begin
	string	cval, cimg, fimg, csub, cstr
	real	rmean

	# Initialize variables...
	cval  = hdrcom
	cimg  = image
	fimg  = flat
	csub  = subset
	rmean = INDEF
	if ( flatlog == "" ) flatlog = "flatproc.log"

	# Clean-up any relevant debris...
	if ( access("ftcor.head") ) delete("ftcor.head", yes, verify-)

	# If first image in list, open flatlog and start logging...
	if (first) {
	   sysinfo() ; gdate(utc=yes)
	   if ( access(flatlog) ) {
	      print ("\n## CCD48PROC (FTCOR48)  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n", >> flatlog)
	   } else {
	      print ("## CCD48PROC (FTCOR48)  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n", >> flatlog)
	   }
	}

	# Check whether image was processed already...
	if ( cval == "COMPLETE" ) {
	   if (verbose) print ("    FLATCOR:  image was processed already; no action taken")
	   print ("  FLATCOR = \"COMPLETE\"; no action taken", >> logfile)
	   print (cimg//": FLATCOR = \"COMPLETE\"; no action taken", >> flatlog)

	} else if ( cval == "OMIT" ) {
	   if (verbose) print ("    FLATCOR = \"OMIT\"; no action taken")
	   print ("  FLATCOR = \"OMIT\"; no action taken", >> logfile)
	   print (cimg//": FLATCOR = \"OMIT\"; no action taken", >> flatlog)

	   # Add header keyword FLATCOR...
	   gdate(utc=yes)
	   hedit (cimg, "flatcor", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "ftcor.head")
	   printf("FLATCOR = %-21s / High-order flat field correction flag\n", "'OMIT'", >> "ftcor.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "ftcor.head")
	   mkheader (cimg, "ftcor.head", append-, verbose-)
	   delete ("ftcor.head", yes, verify-)

	} else {
	   if ( verbose && !first ) printf("    FLATCOR ...")

	   # Test whether appropriate flatfield is present...
	   chkimg (fimg//csub, "access", imtype="", verbose-)
	   fimg = chkimg.impath//chkimg.imroot
	   if (!chkimg.ok) {
	      print ("    FLATCOR ...  flat \""//fimg//chkimg.imtype//"\" not found; no action taken")
	      print ("  FLATCOR:  flat image \""//fimg//chkimg.imtype//"\" not found; no action taken", >> logfile)
	      print (cimg//": flat image \""//fimg//chkimg.imtype//"\" not found; no action taken", >> flatlog)
	      return
	   }

	   # Test whether flat frame has mean=1; if not, (re)normalize...
	   if (first) {
	      imstat (fimg, fields="mean", lower=INDEF, upper=INDEF,
			nclip=3., lsigma=4., usigma=4., binwidth=0.1,
			format-, cache-) | scan(rmean)

	      if ( rmean == INDEF || rmean == 0. ) rmean = 1.
	      if ( abs(rmean-1.) > 1e-3 ) {
		 imarith (fimg, "/", rmean, fimg, title="", divzero=0.,
			hparams="", pixtype="real", calctype="real",
			verbose-, noact-)

		 # Add keywords NORMALIZ, NMLZMEAN, and NMLZDATE to image header
		 if ( access("fnorm.head") ) delete ("fnorm.head", yes, verify-)
		 gdate(utc=yes)
		 hedit (fimg, "normaliz", "", add-, del+, ver-, show-, upd+)
		 hedit (fimg, "nmlzmean", "", add-, del+, ver-, show-, upd+)
		 hedit (fimg, "nmlzdate", "", add-, del+, ver-, show-, upd+)
		 hedit (fimg, "cc48date", "", add-, del+, ver-, show-, upd+)
		 imheader (fimg, longheader+, > "fnorm.head")
		 printf("NORMALIZ= %-21s / Flat field normalization flag\n", >> "fnorm.head")
		 printf("NMLZMEAN= %20.4f  / Mean level (ADU) prior to normalization\n", rmean, >> "fnorm.head")
		 print ("NMLZDATE= '"//gdate.fdate//"' / Flat field normalization done", >> "fnorm.head")
		 print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fnorm.head")
		 mkheader (fimg, "fnorm.head", append-, verbose-)
		 delete ("fnorm.head", yes, verify-)

		 # Log action to text log files and screen...
		 print ("    FLATNORM ...")
		 printf("       Renormalizing \""//fimg//"\" (mean=%.4f-->1.0000) ...\n", rmean)
		 printf("  FLATNORM: renormalized \""//fimg//"\" (mean=%.4f-->1.0000)\n", rmean, >> logfile)
		 printf("Renormalized \""//fimg//"\" (mean=%.4f-->1.0000) ...\n", rmean, >> flatlog)
	      }

	      if (verbose) printf("    FLATCOR ...")
	   }

	   # Divide image by the appropriate flatfield...
	   imarith (cimg, "/", fimg, cimg, title="", divzero=0., hparams="",
		pixtype="real", calctype="real", verbose+, noact-, >> flatlog)

	   # Add keywords FLATCOR, FTCRIMAG and FTCRDATE to image header...
	   gdate(utc=yes)
	   hedit (cimg, "flatcor", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "ftcrimag", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "ftcrdate", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "ftcor.head")
	   printf("FLATCOR = %-21s / High-order flat field correction flag\n", "'COMPLETE'", >> "ftcor.head")
	   if ( strlen(fimg) <= 21 ) {
	      cstr = "'"//fimg//"'"
	      printf("FTCRIMAG= %-21s / Name of high-order flat image\n", cstr, >> "ftcor.head")
	   } else {
	      if ( strlen(fimg) <= 36 ) {
		 cstr = "'"//fimg//"'"
	      } else {
		 cstr = "'..."//substr(fimg,(strlen(fimg)-32),strlen(fimg))//"'"
	      }
	      printf("FTCRIMAG= %-"//strlen(cstr)//"s / Name of high-order flat image\n", cstr, >> "ftcor.head")
	   }
	   print ("FTCRDATE= '"//gdate.fdate//"' / High-order flatfielding done", >> "ftcor.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "ftcor.head")
	   mkheader (cimg, "ftcor.head", append-, verbose-)
	   delete ("ftcor.head", yes, verify-)

	   if (verbose) printf("  done (subs=\""//csub//"\")\n")

	   # Log action to text log files...
	   print ("  FLATCOR:  "//gdate.fdate//" Flat correction using image "//cstr, >> logfile)
	   print (cimg//":", >> flatlog)
	   print ("  FLATCOR:  "//gdate.fdate//" Flat correction using image "//cstr, >> flatlog)

	}

end
