procedure ovsca48 (hdrcom, image, biassec, trim, trimsec, ncol, nrow, logfile,
		zerolog, verbose, first)

string	hdrcom	=""		{prompt="Header controled command"}
string	image	=""		{prompt="Image name"}
string	biassec	=""		{prompt="Image overscan section"}
bool	trim	=yes		{prompt="Trim image?"}
string	trimsec	=""		{prompt="Image data section"}
int	ncol	=INDEF		{prompt="Number of columns"}
int	nrow	=INDEF		{prompt="Number of lines"}
string	logfile	=""		{prompt="Text logfile ccd48proc"}
string	zerolog	=""		{prompt="Text logfile biassubtraction"}
bool	verbose	=yes		{prompt="Verbose messages?"}
bool	first	=no		{prompt="First image in list?"}
string	imagsec	=""		{prompt="Image section after trimming"}

# @(#) subtask ovsca48            Author: R.A. Jansen -- January 17 1996
# @(#) 
# @(#) subtask called from 'ccd48proc' to subtract the bias level as de-
# @(#) termined in the overscan section and, optionally, trim the image.
# @(#)
# @(#) Aug 29 2002 -- Headers are now updated  using  'artdata.mkheader'
# @(#)   rather than 'ccdhedit'. Keywords are now 'OVERSCAN' (with value
# @(#)   'PERFORM', 'OMIT', or  'COMPLETE'),  'BIASSEC',  'TRIMSEC'  and
# @(#)   'IMAGSEC',  the mean overscan level subtracted ('OVSCMEAN') and
# @(#)   'OVSCDATE' (FITS Y2K compliant date + time string). RAJ.
# @(#) Aug 14 2003 -- Added 'hdrcom' to support header keyword controled
# @(#)   data reduction (a la 'calstis') and added 'verbose' switch. RAJ

begin
	string	cval, cimg, bsec, tsec, isec, osec, ctv
	real	rlo, rhi, bias, ltv1, ltv2
	int	nc, nr
	bool	trm

	# Initialize variables...
	cval  = hdrcom
	cimg  = image
	bsec  = biassec
	trm   = trim
	tsec  = trimsec
	isec  = tsec
	nc    = ncol
	nr    = nrow
	if ( zerolog == "" ) zerolog = "biasproc.log"

	# Clean-up any relevant debris...
	if ( access("ovsca.head") ) delete("ovsca.head", yes, verify-)
	if ( access("_ovsca.log") ) delete("_ovsca.log", yes, verify-)

	# If first image in list, open zerolog and start logging...
	if (first) {
	   sysinfo() ; gdate(utc=yes)
	   if ( access(zerolog) ) {
	      print ("\n## CCD48PROC (OVSCA48)  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n", >> zerolog)
	   } else {
	      print ("## CCD48PROC (OVSCA48)  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n", >> zerolog)
	   }
	}

	# Check whether image was processed already...
	if ( cval == "COMPLETE" ) {
	   if (verbose) print ("    OVERSCAN: image was processed already; no action taken")
	   print ("  OVERSCAN= \"COMPLETE\"; no action taken", >> logfile)
	   print (cimg//": OVERSCAN= \"COMPLETE\"; no action taken", >> zerolog)

	} else if ( cval == "OMIT" ) {
	   if (verbose) print ("    OVERSCAN= \"OMIT\"; no action taken")
	   print ("  OVERSCAN= \"OMIT\"; no action taken", >> logfile)
	   print (cimg//": OVERSCAN= \"OMIT\"; no action taken", >> zerolog)

	   # Add header keyword OVERSCAN...
	   gdate(utc=yes)
	   hedit (cimg, "overscan", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "ccdtrim", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "ovsca.head")
	   printf("OVERSCAN= %-21s / Overscan subtraction flag\n", "'OMIT'", >> "ovsca.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "ovsca.head")
	   mkheader (cimg, "ovsca.head", append-, verbose-)
	   delete ("ovsca.head", yes, verify-)

	} else {
	   if (verbose) printf("    OVERSCAN ... ")

	   # measure mean level in overscan strip...
	   imgstat (cimg//bsec, lower=INDEF, upper=INDEF, verbose-)
	   rlo = imgstat.midpt - 4*imgstat.stddev
	   rhi = imgstat.midpt + 4*imgstat.stddev
	   imgstat (cimg//bsec, lower=rlo, upper=rhi, binwidth=0.1, verbose-)
	   bias = imgstat.mean

	   # default output section when no trimming is desired...
	   if (!trm) tsec="[1:"//str(nc)//",1:"//str(nr)//"]"

	   #  bias subtract input image...
	   rpparse (bsec, verbose+)
	   if ( rpparse.xcen < 0.1*nc || rpparse.xcen > 0.9*nc ) {
	      colbias (cimg, cimg, bias=bsec, trim=tsec, median+,
			interactive-, function="legendre", order=1,
			low_reject=3., high_reject=3., niterate=3,
			logfiles="_ovsca.log", graphics="stdgraph", cursor="")
	   } else {
	      linebias(cimg, cimg, bias=bsec, trim=tsec, median+,
			interactive-, function="legendre", order=1,
			low_reject=3., high_reject=3., niterate=3,
			logfiles="_ovsca.log", graphics="stdgraph", cursor="")
	   }
	   printf("\n", >> "_ovsca.log")
	   if (verbose) printf("done (bias=%8f ADU)\n", bias)

	   # Add keywords OVERSCAN, *SEC, OVSCMEAN and OVSCDATE, and
	   # update the IRAF WCS keywords...
	   gdate(utc=yes)
	   hedit (cimg, "overscan", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "ccdtrim", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "ovscmean", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "ovscdate", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "ltv*", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, lo+) | match ("*SEC", "-", stop+, > "ovsca.head")
	   printf("OVERSCAN= %-21s / Overscan subtraction flag\n", "'COMPLETE'", >> "ovsca.head")

	   rpparse (tsec, verbose+)
	   ltv1 = real(1.0-rpparse.xlo)
	   ltv2 = real(1.0-rpparse.ylo)
	   rpbuild (1, (rpparse.xhi-rpparse.xlo+1), 1,
			(rpparse.yhi-rpparse.ylo+1), pln=0)
	   isec = rpbuild.imrpstr
	   rpbuild (1, nc, 1, nr, pln=0)
	   osec = rpbuild.imrpstr

	   printf("ORIGSEC = %-21s / Original size of full frame\n", "'"//osec//"'", >> "ovsca.head")
	   printf("BIASSEC = %-21s / Overscan section\n", "'"//bsec//"'", >> "ovsca.head")
	   printf("TRIMSEC = %-21s / Extraction section\n", "'"//tsec//"'", >> "ovsca.head")
	   printf("IMAGSEC = %-21s / Output image section\n", "'"//isec//"'", >> "ovsca.head")
	   #ctv = "" ; hselect (cimg, "ltv1", "yes") | scan (ctv)
	   #if ( ctv == "" ) {
	   printf("LTV1    = %20.0f  / Translation wrt to original image, axis 1\n", ltv1, >> "ovsca.head")
	   #}
	   #ctv = "" ; hselect (cimg, "ltv2", "yes") | scan (ctv)
	   #if ( ctv == "" ) {
	   printf("LTV2    = %20.0f  / Translation wrt to original image, axis 2\n", ltv2, >> "ovsca.head")
	   #}
	   printf("OVSCMEAN= %20.2f  / Mean bias level (ADU)\n", bias, >> "ovsca.head")
	   print ("OVSCDATE= '"//gdate.fdate//"' / Removal bias level using overscan done", >> "ovsca.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "ovsca.head")
	   mkheader (cimg, "ovsca.head", append-, verbose-)
	   delete ("ovsca.head", yes, verify-)

	   #hedit (cimg, "ltv1", int(ltv1), add-, del-, ver-, show-, upd+)
	   #hedit (cimg, "ltv2", int(ltv2), add-, del-, ver-, show-, upd+)

	   # Log action to text log files...
	   printf("  OVERSCAN: "//gdate.fdate//" Subtracted overscan level: mean=%8f\n", bias, >> logfile)
	   chkimg (cimg, "access", imtype="", verbose-)
	   print (chkimg.impath//chkimg.imroot//chkimg.imtype//":", >> zerolog)
	   print ("  overscan strip = "//bsec, >> zerolog)
	   print ("  trim section = "//tsec, >> zerolog)
	   match ("functi", "_ovsca.log", stop-) | translit ("", "#", " ", del-, col-, >> zerolog)
	   match ("order", "_ovsca.log", stop-) | translit ("", "#", " ", del-, col-, >> zerolog)
	   match ("reject", "_ovsca.log", stop-) | translit ("", "#", " ", del-, col-, >> zerolog)
	   match ("niter", "_ovsca.log", stop-) | translit ("", "#", " ", del-, col-, >> zerolog)
	   printf("  mean = %8f\n", bias, >> zerolog)
	   match ("RMS", "_ovsca.log", stop-) | translit ("", "#", " ", del-, col-, >> zerolog)
	   match ("chi", "_ovsca.log", stop-) | translit ("", "#", " ", del-, col-, >> zerolog)
	   print ("OVERSCAN: "//gdate.fdate//" Subtracted overscan level\n", >> zerolog)
	   delete ("_ovsca.log", yes, verify-)

	   # Return the image section after trimming...
	   imagsec = isec
	}

end
