procedure sccor48 (hdrcom, image, scat, subset, trimsec, logfile, verbose)

string	hdrcom	=""		{prompt="Header controled command"}
string	image	=""		{prompt="Image name"}
string	scat	=""		{prompt="Scattered light correction image"}
string	subset	=""		{prompt="Image subset"}
string	trimsec	=""		{prompt="CCD trim section"}
string	logfile	=""		{prompt="Text logfile ccd48proc"}
bool	verbose	=yes		{prompt="Verbose messages?"}

# @(#) subtask sccor48           Author: R.A. Jansen -- February 03 1996
# @(#) 
# @(#) subtask called from  'ccd48proc'  to correct  for scattered light
# @(#) using a scattered light calibration image.
# @(#)
# @(#) Nov  3 2002 -- Headers are now updated  using  'artdata.mkheader'
# @(#)   rather than 'ccdhedit'.  Keywords are now 'SCATCOR' (with value
# @(#)   'PERFORM','OMIT', or 'COMPLETE'), 'SCCRIMAG' with image name as
# @(#)   its value, and 'SCCRDATE' (FITS Y2K compliant date+time string).
# @(#) Aug 14 2003 -- Added 'hdrcom' to support header keyword controled
# @(#)   data reduction (a la 'calstis') and added 'verbose' switch. RAJ

begin
	string	cval, cimg, simg, csub, cstr
	int	icol, irow
        real	rlo, rhi, rmed

	# Initialize variables...
	cval  = hdrcom
	cimg  = image
	simg  = scat
	csub  = subset

	# Clean-up any relevant debris...
	if ( access("sccor.head") ) delete("sccor.head", yes, verify-)

	# Check whether image was processed already...
	if ( cval == "COMPLETE" ) {
	   if (verbose) print ("    SCATCOR:  image was processed already; no action taken")
	   print ("  SCATCOR = \"COMPLETE\"; no action taken", >> logfile)

	} else if ( cval == "OMIT" ) {
	   if (verbose) print ("    SCATCOR = \"OMIT\"; no action taken")
	   print ("  SCATCOR = \"OMIT\"; no action taken", >> logfile)

	   # Add header keyword SCATCOR...
           gdate(utc=yes)
           hedit (cimg, "scatcor", "", add-, del+, ver-, show-, upd+)
           hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
           imheader (cimg, longheader+, > "sccor.head")
           printf("SCATCOR = %-21s / Scattered light correction flag\n", "'OMIT'", >> "sccor.head")
           print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "sccor.head")
           mkheader (cimg, "sccor.head", append-, verbose-)
           delete ("sccor.head", yes, verify-)

	} else {
	   # Test whether appropriate scattered light frame is present...
	   chkimg (simg//csub, "access", imtype="", verbose-)
	   simg = chkimg.impath//chkimg.imroot//csub
	   if (!chkimg.ok) {
	      print ("    SCATCOR ... scattered light image \""//simg//chkimg.imtype//"\" not found; no action taken")
	      print ("  SCATCOR:  scattered light image \""//simg//chkimg.imtype//"\" not found; no action taken", >> logfile)
	      return
	   }

	   if (verbose) print ("    SCATCOR ... subs=\""//csub//"\"")

	   # Compute midpoint of pixel values in a 25 pixel wide border...
	   borderstat (cimg, 25, lower=INDEF, upper=INDEF, binwidth=0.1, verb-)
	   rlo = borderstat.midpt - 3*borderstat.stddev
	   rhi = borderstat.midpt + 3*borderstat.stddev
	   borderstat (cimg, 25, lower=rlo, upper=rhi, binwidth=0.1, verbose-)
	   rmed = borderstat.midpt

	   # Scale and subtract the scattered light calibration image...
	   imarith (simg, "*", rmed, "tmpsct.fits", title="", divzero=0.,
		hparams="", pixtype="real", calctype="real", verb-, noact-)
	   imarith (cimg, "-", "tmpsct.fits", cimg, title="", divzero=0.,
		hparams="", pixtype="real", calctype="real", verb-, noact-)
	   delete ("tmpsct.fits", yes, verify-)

	   # Add keywords SCATCOR, SCCRIMAG and SCCRDATE to image header...
	   gdate(utc=yes)
	   hedit (cimg, "scatcor", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "sccrimag", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "sccrdate", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "sccor.head")
	   printf("SCATCOR = %-21s / Scattered light correction flag\n", "'COMPLETE'", >> "sccor.head")
	   if ( strlen(simg) <= 21 ) {
	      cstr = "'"//simg//"'"
	      printf("SCCRIMAG= %-21s / Name of scattered light image\n", cstr, >> "sccor.head")
	   } else {
	      if ( strlen(simg) <= 47 ) {
		 cstr = "'"//simg//"'"
	      } else {
		 cstr = "'..."//substr(simg,(strlen(simg)-43),strlen(simg))//"'"
	      }
	      printf("SCCRIMAG= %-"//strlen(cstr)//"s / Name of scattered light image\n", cstr, >> "sccor.head")
	   }
	   print ("SCCRDATE= '"//gdate.fdate//"' / Illumination correction done", >> "sccor.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "sccor.head")
	   mkheader (cimg, "sccor.head", append-, verbose+)
	   delete ("sccor.head", yes, verify-)

	   # Log action to text log file...
	   print ("  SCATCOR:  "//gdate.fdate//" Scattered light correction using image "//cstr, >> logfile)

	}

end
