procedure scmod48 (imagetyp,npix1,npix2)

string	imagetyp		{prompt="Image type"}
int	npix1			{prompt="Number of image columns"}
int	npix2			{prompt="Number of image lines (rows)"}
string	logfile	="logfile"	{prompt="Text logfile ccd48proc"}
bool	verbose	=yes		{prompt="Verbose?"}
bool	ok	=yes		{prompt="No errors encountered?"}

# @(#) subtask scmod48           Author: R.A. Jansen -- February 02 1996
# @(#) 
# @(#) subtask called from 'ccd48proc' to create a correction image for
# @(#) scattered light falling onto the CCD using a (combined) flatfield
# @(#) image. 
# @(#)
# @(#) NB: needs to be updated for image formats other than OIF !

begin
	string	img, imgtyp, filter, dstr, logstr, scat
        real	mmode
        int	nrow, ncol, imglen, bd, Nflats, NS

        imgtyp = imagetyp
        nrow = npix2
        ncol = npix1
        bd = 25
        ok=yes

#       List the combined flats that are present in current directory ...
        print("!\\ls "//imgtyp//"?.imh > "//imgtyp//".scatcor", >> "tmpsccr.cl")
        cl < tmpsccr.cl
        delete ("tmpsccr.cl")

#       Count the number of combined flats to loop over ...
        iwc (imgtyp//".scatcor", nlines=0, nwords=0, nchars=0)
        Nflats = iwc.nwords

#       Loop over all combined flats ...
        NS = 1
        if (verbose) { 
           print(" ")
           print("CCD48PROC scattered light modelling:")
        }
        while ( NS <= Nflats ) {

           rdlist (imgtyp//".scatcor", NS, cline="")
           img = rdlist.cline

           imgets (img, "filter") ; filter = imgets.value
           imgets (img, "filter") ; filter = imgets.value
           imglen = strlen(img)
           if ( substr(img,imglen-3,imglen) == ".imh" ) { imglen = imglen - 4 }

           if (verbose) { 
              print("    SCATMOD  Input="//substr(img,1,imglen)//".imh")
              if ( imgtyp == "FLAT" ) {
                 print("      FITRESP   Result="//substr(img,1,imglen)//".fit.imh")
              }
              else {
                 print("      FITILLUM  Result="//substr(img,1,imglen)//".fit.imh")
              }
           }

#####   NB: THE FOLLOWING FITTING STEP IS  E X T R E M E L Y  TIME CONSUMING !!

#          Fit the light distibution and create a respons frame
           imsurfit (img, substr(img,1,imglen)//".fit", xorder=32, yorder=32,
                  type_output="fit", function="legendre", cross_terms=yes,
                  xmedian=1, ymedian=1, median_perce=50., lower=3., upper=3.,
                  ngrow=1, niter=2, regions="all", rows="*", columns="*",
                  border="50", sections="", circle="", div_min=INDEF)



#          Normalize the illumination image
           borderstat (img, ncol, nrow, bd, domode=yes)
           mmode = borderstat.rmode

#          Create the illumination/respons frame with the pixel to pixel
#          variations and CCD defects still present, but without the
#          scattered light
           if ( imgtyp == "FLAT" ) {
              if (verbose) {
                 print("      NORMALIZE Result=RESP"//filter//".imh")
              }
              imarith (img, "/", substr(img,1,imglen)//".fit", "RESP"//filter,
                      title="", divzero=0., hparams="", pixtype="real", 
                      calctype="real", verbose=no, noact=no)

#             Add header keyword SCATCOR and log info to image header
              gdate(); dstr = gdate.date
              print ("RESP"//filter//": "//dstr//" Scatt.light correction using image "//substr(img,1,imglen)//".fit", >> logfile)
              logstr = dstr//" Scatt.light correction image="//substr(img,1,imglen)//".fit"
              ccdhedit ("RESP"//filter, "scatcor", logstr, type="string")
#             Add also header keyword NORMALIZ and log info to image header
              print ("RESP"//filter//": "//dstr//" Flatfield normalization, mean="//str(mmode), >> logfile)
              logstr = dstr//" Flatfield normalization, mean="//str(mmode)
              ccdhedit ("RESP"//filter, "normaliz", logstr, type="string")

              scat = "SCATCORF"
           }
           else {
              if (verbose) { 
                 print("      NORMALIZE Result=ILLUM"//filter//".imh") 
              }
              imarith (img, "/", substr(img,1,imglen)//".fit", "ILLUM"//filter,
                      title="", divzero=0., hparams="", pixtype="real", 
                      calctype="real", verbose=no, noact=no)

#             Add header keyword SCATCOR and log info to image header
              gdate(); dstr = gdate.date
              print ("ILLUM"//filter//": "//dstr//" Scatt.light correction using image "//substr(img,1,imglen)//".fit", >> logfile)
              logstr = dstr//" Scatt.light correction image="//substr(img,1,imglen)//".fit"
              ccdhedit ("ILLUM"//filter, "scatcor", logstr, type="string")
#             Add also header keyword NORMALIZ and log info to image header
              print ("ILLUM"//filter//": "//dstr//" Skyflat normalization, mean="//str(mmode), >> logfile)
              logstr = dstr//" Skyflat normalization, mean="//str(mmode)
              ccdhedit ("ILLUM"//filter, "normaliz", logstr, type="string")

              scat = "SCATCOR"
           }


#          Calculate the mean of the fit image in a 25 pix wide border
           borderstat (img, ncol, nrow, bd)
           mmode = borderstat.mean

#          Divide the fit image by this mean an subtract 1, to obtain a 
#          scale-less correction image
           if (verbose) { 
              print("      CREASCAT  Result="//scat//filter//".imh")
           }
           imarith (substr(img,1,imglen)//".fit", "/", mmode, scat//filter,
                    title="", divzero=0., hparams="", pixtype="real", 
                    calctype="real", verbose=no, noact=no)
           imarith (scat//filter, "-", 1.0000, scat//filter, title="", 
                    divzero=0., hparams="", pixtype="real", calctype="real",
                    verbose=no, noact=no)

#          Add header keyword CREASCAT and log info to image header
           gdate(); dstr = gdate.date
           print (scat//filter//": "//dstr//" Created scatcor image using "//substr(img,1,imglen), >> logfile)
           logstr = dstr//" Created scatcor image using "//substr(img,1,imglen)
           ccdhedit (scat//filter, "creascat", logstr, type="string")

           NS += 1

#       End of while-loop
        }

        print(" ")

end
