procedure wcsin48 (hdrcom, image, ncol, nrow, logfile, transp, verbose)

string	hdrcom	=""		{prompt="Header controled command"}
string	image	=""		{prompt="Image name"}
int	ncol	=INDEF		{prompt="Number of columns"}
int	nrow	=INDEF		{prompt="Number of lines"}
string	logfile	=""		{prompt="Text logfile"}
bool	transp	=no		{prompt="Transpose to N up and E left?"}
bool	verbose =yes		{prompt="Verbose messages?"}

# @(#) subtask wcsin48           Author: R.A. Jansen -- November  4 2002
# @(#) 
# @(#) sub-task called from 'ccd48proc'  to initialize the WCS keywords.
# @(#) Keywords are 'WCSDIM', 'LTV*', 'LTM*_*', and 'WAT*_001'  and also
# @(#) 'WCSINIT' with values 'PERFORM', 'OMIT', or 'COMPLETE' as well as
# @(#) 'WCSDATE' (FITS Y2K compliant date + time string).
# @(#)
# @(#) Aug 13 2003 -- Added  'transp'  switch  to pass wether the CCD is
# @(#)   front- (no) or back-side (yes) illuminated and, hence,  whether
# @(#)   the pixel coordinates are transposed with respect to the celes-
# @(#)   tial axes; added 'hdrcom' to support header keyword controled
# @(#)   data reduction (a la 'calstis') and added 'verbose' switch. RAJ
# @(#) Jul  2 2004 -- Allowed RA, DEC  coordinate strings  in headers to
# @(#)   have embedded spaces  by first passing the output of  'hselect'
# @(#)   through 'translit'. RAJ.

begin
	string	cval, cimg, dash
	int	nc, nr, ld
	real	rra, rdec, scpx
	bool	btr

	# Initialize variables...
	cval = hdrcom
	cimg = image
	nc   = ncol
	nr   = nrow
	btr  = transp
	dash = "------------"

	# Clean-up any possible debris from previous runs...
	if ( access("wcsin.head") ) delete ("wcsin.head", yes, verify-)

	# Check whether image was processed already...
	if ( cval == "COMPLETE" ) {
	   if (verbose) print ("    WCSINIT:  header is up to date; no action taken")
	   print ("  WCSINIT = \"COMPLETE\"; no action taken", >> logfile)

	} else if ( cval == "OMIT" ) {
	   if (verbose) print ("    WCSINIT = \"OMIT\"; no action taken")
	   print ("  WCSINIT = \"OMIT\"; no action taken", >> logfile)

	   # Add header keyword WCSINIT...
	   gdate(utc=yes)
	   hedit (cimg, "wcsinit", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "wcsin.head")
	   printf("WCSINIT = %-21s / WCS initialization flag\n", "'OMIT'", >> "wcsin.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "wcsin.head")
	   mkheader (cimg, "wcsin.head", append-, verbose-)
	   delete ("wcsin.head", yes, verify-)

	} else {
	   if (verbose) printf("    WCSINIT ...")
	   # Determine nominal pointing coords...
	   hselect(cimg, "ra", "yes") | translit("-", "\"", "", del+) | scan(rra)
	   hselect(cimg, "dec", "yes")| translit("-", "\"", "", del+) | scan(rdec)
	   rra  = 15.*rra
	   scpx = (getid48.secpix/3600.)

	   # Initialize WCS header keywords...
	   gdate(utc=yes)
	   hedit (cimg, "crval*", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "crpix*", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cdelt*", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "ctype*", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cd1_1", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cd2_2", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "ltv1", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "ltv2", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "ltm1_1", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "ltm2_2", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "wat*001", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "wcsinit", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "wcsdim", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+) | match ("CC48PROC", "-", stop+) | match ("HISTORY", "-", stop+, > "wcsin.head")
	   printf("CRVAL1  = %20.6f / RA at reference pixel in degrees\n", rra, >> "wcsin.head")
	   printf("CRPIX1  = %20.1f / Reference pixel, axis 1\n", 0.5*nc, >> "wcsin.head")
	   if (btr) {
	      printf("CDELT1  = %20.8f / Increment per pixel on axis1, in degrees\n", scpx, >> "wcsin.head")
	   } else {
	      printf("CDELT1  = %20.8f / Increment per pixel on axis1, in degrees\n", (-1.0*scpx), >> "wcsin.head")
	   }
	   printf("CTYPE1  = %-20s / Coordinate system of axis1\n", "'RA---TAN'", >> "wcsin.head")
	   printf("CRVAL2  = %20.5f / DEC at reference pixel in degrees\n", rdec, >> "wcsin.head")
	   printf("CRPIX2  = %20.1f / Reference pixel, axis 2\n", 0.5*nr, >> "wcsin.head")
	   if (btr) {
	      printf("CDELT2  = %20.8f / Increment per pixel on axis2, in degrees\n", (-1.0*scpx), >> "wcsin.head")
	   } else {
	      printf("CDELT2  = %20.8f / Increment per pixel on axis2, in degrees\n", scpx, >> "wcsin.head")
	   }
	   printf("CTYPE2  = %-20s / Coordinate system of axis2\n", "'DEC--TAN'", >> "wcsin.head")
	   if (btr) {
	      printf("CD1_1   = %20.11e / Diagonal element of scale matrix (1,1)\n", scpx, >> "wcsin.head")
	      printf("CD2_2   = %20.11e / Diagonal element of scale matrix (2,2)\n", (-1.0*scpx), >> "wcsin.head")
	   } else {
	      printf("CD1_1   = %20.11e / Diagonal element of scale matrix (1,1)\n", (-1.0*scpx), >> "wcsin.head")
	      printf("CD2_2   = %20.11e / Diagonal element of scale matrix (2,2)\n", scpx, >> "wcsin.head")
	   }
	   printf("LTV1    = %20d / Translation wrt to original image, axis 1\n", 0, >> "wcsin.head")
	   printf("LTV2    = %20d / Translation wrt to original image, axis 2\n", 0, >> "wcsin.head")
	   printf("LTM1_1  = %20d / Rotation wrt to original image, M(1,1)\n", 1, >> "wcsin.head")
	   printf("LTM2_2  = %20d / Rotation wrt to original image, M(2,2)\n", 1, >> "wcsin.head")
	   printf("WAT0_001= %-20s / WCS attribute\n", "'system=world'", >> "wcsin.head")
	   printf("WAT1_001= %-20s / WCS attribute, axis 1\n", "'wtype=linear axtype=ra'", >> "wcsin.head")
	   printf("WAT2_001= %-20s / WCS attribute, axis 2\n", "'wtype=linear axtype=dec'", >> "wcsin.head")
	   print ("HISTORY -", >> "wcsin.head")
	   ld = nint(0.5*(37-strlen(sysinfo.username)-strlen(sysinfo.host)))
	   print ("HISTORY "//substr(dash,1,ld)//"  CCD48PROC["//sysinfo.version//"]  "//sysinfo.username//"@"//sysinfo.host//"  "//substr(dash,1,ld), >>"wcsin.head")
	   print ("HISTORY -", >> "wcsin.head")
	   imheader (cimg, longheader+) | match ("CC48PROC", "-", stop-, >> "wcsin.head")
	   printf("WCSINIT = %-21s / WCS initialization flag\n", "'COMPLETE'", >> "wcsin.head")
	   printf("WCSDIM  = %20d  / WCS dimension\n", 2, >> "wcsin.head")
	   print ("WCSDATE = '"//gdate.fdate//"' / WCS initialization done", >> "wcsin.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "wcsin.head")
	   mkheader (cimg, "wcsin.head", append-, verbose-)
	   delete ("wcsin.head", yes, verify-)

	   if (verbose) printf("  done\n")

	   # Log action to text log file...
	   print ("  WCSINIT:  "//gdate.fdate//" WCS initialization done (transp="//btr//")", >> logfile)

	}

end
